// -------------------------------------------------------------------------\\
//          
// File:        cell.h
// Author:      Bettina McCormick
//          
// Project:     Atlantis: COP 2224 VCC; Summer 1999
//          
// Created:     June 18, 1999
// Updated:     July 21, 1999
// Date due:    August 04, 1999
//             
// Description: Specification for type Cell
//
//              A Cell is a piece of a Lostcity (Lostcity "has a" Cell)
//              A Cell "has a" Target
//        
// Notes:       Turn on symbol CELL_TEST to test this component
//
// -------------------------------------------------------------------------// 
      
#ifndef CELL_H        // standard header wrapper
#define CELL_H
    
#include "target.h"
#include "pod.h"
#include "crystal.h"
    
namespace Atlantis    // project namespace
{
  
  class Cell      // not derived from any other class
  {
  public:
    Cell ();
    /**------------------------------------------------------------------------
    // public constructor: default - no arguments
    //-----------------------------------------------------------------------*/
      
    ~Cell();
    /**------------------------------------------------------------------------
    // destructor
    //-----------------------------------------------------------------------*/
      
    void  Set_xyz ( int x_row=0, int y_col=0, int z_dep=0 );
    /**------------------------------------------------------------------------
    // mutator method: sets all three coordinates of cell
    //-----------------------------------------------------------------------*/
      
    int   Get_x ( void ) const;
    /**------------------------------------------------------------------------
    // accessor method: returns x_row coordinate (row)
    //-----------------------------------------------------------------------*/
      
    int   Get_y ( void ) const;
    /**------------------------------------------------------------------------
    // accessor method: returns y_col coordinate (row)
    //-----------------------------------------------------------------------*/
    
    int   Get_z ( void ) const;
    /**------------------------------------------------------------------------
    // accessor method: returns z_dep coordinate (row)
    //-----------------------------------------------------------------------*/
        
    bool  Set_target ( Target * t );
    /**------------------------------------------------------------------------
    // mutator method: assigns target to Cell
    // returns true if cell was emtpy and could be assigned a target;
    // returns false if cell wasd already occupied by a target
    //-----------------------------------------------------------------------*/
        
    Target * Get_target ( void ) const;
    /**------------------------------------------------------------------------
    // accessor method: returns ponter to the target cell contains
    //-----------------------------------------------------------------------*/
          
    bool  Hit ( void );
    /**------------------------------------------------------------------------
    // polymorphically calls Hit of the derived target object through the 
    // pointer to the target
    //-----------------------------------------------------------------------*/
          
    void ShowMiss ( void ) const;
    /**------------------------------------------------------------------------
    // shows a "miss" - no target in cell to hit
    //-----------------------------------------------------------------------*/
   
  private:
    Cell ( const Cell & c );  // copy constructor
    bool hit_status;          // true if the cell is hit
    Target * target;          // assigned to the cell
    int x_row;                // row position in the lostcity     
    int y_col;                // column position in the lostcity
    int z_dep;                // depth position in the lostcity
  }; // end class Cell
  
} // namepsace Atlantis

#endif // CELL_H

// eof cell.h

