//          
// File:        lostcity.h
// Author:      Bettina McCormick
//          
// Project:     Atlantis: COP 2224 VCC; Summer 1999
//          
// Created:     July 08, 1999
// Updated:     July 28, 1999
// Date due:    August 04, 1999
//             
// Description: Specification for type Lostcity
//
//              Lostcity is a container for cells
//        
// Notes:       Turn on symbol LOSTCITY_TEST to test this component
//    
// -------------------------------------------------------------------------// 

#ifndef LOSTCITY_H        // standard header wrapper
#define LOSTCITY_H

#include <string>

#include "constuff.h"
#include "cell.h"
#include "target.h"
#include "pod.h"
#include "crystal.h"

namespace Atlantis        // project namespace
{
  
  class Lostcity
  {
  public:
    Lostcity();
    /**------------------------------------------------------------------------
    // public constructor: default - no arguments
    // prompts user for Lostcity name
    //-----------------------------------------------------------------------*/
        
    Lostcity( const std::string & name );
    /**------------------------------------------------------------------------
    // overloaded constructor: takes reference to string for name
    //-----------------------------------------------------------------------*/
    
    ~Lostcity();
    /**------------------------------------------------------------------------
    // destructor - when lost city is destructed, the objects in the city must 
    // also be destroyed
    //-----------------------------------------------------------------------*/
        
    std::string Get_name( void ) const;
    /**------------------------------------------------------------------------
    // accessor method: returns lostcity name
    //-----------------------------------------------------------------------*/

    int Get_target_count( void ) const;
    /**------------------------------------------------------------------------
    // accessor method: returns target count
    //-----------------------------------------------------------------------*/
        
    int Get_active_targets( void ) const;
    /**------------------------------------------------------------------------
    // accessor method: returns active targets
    //-----------------------------------------------------------------------*/
        
    int Get_active_pods( void ) const;
    /**------------------------------------------------------------------------
    // accessor method: returns active pods
    //-----------------------------------------------------------------------*/
    
    int Get_destroyed_targets( void ) const;
    /**------------------------------------------------------------------------
    // accessor method: returns destroyed targets
    //-----------------------------------------------------------------------*/
    
    int Get_destroyed_pods( void ) const;
    /**------------------------------------------------------------------------
    // accessor method: returns destroyed pods
    //-----------------------------------------------------------------------*/
    
    int Get_damaged_crystals( void );
    /**------------------------------------------------------------------------
    // goes thru target list, counts how many crystals are damaged & returns
    //-----------------------------------------------------------------------*/
    
    void Set_game_over( bool go );
    /**------------------------------------------------------------------------
    // mutator method: sets game_over
    //-----------------------------------------------------------------------*/
    
    void Set_time_elapsed( bool te );
    //**------------------------------------------------------------------------
    // mutator method: sets time_elapsed
    //-----------------------------------------------------------------------*/
    
    void Set_difficulty_level( int dl );
    /**------------------------------------------------------------------------
    // mutator method: sets difficulty_level
    //-----------------------------------------------------------------------*/
    
    int User_defines_level( void );
    /**------------------------------------------------------------------------
    // prompts user for difficulty level 
    // level equates to number of pods and crystals that will fill the lostcity
    //-----------------------------------------------------------------------*/
    
    void PlaceTarget(  int object, int obj_type );
    /**------------------------------------------------------------------------
    // takes int describing the object (pod or crystal) and int defining type 
    // of pod or crystal; creates respective object & places it in the 
    // "target_list" provided that maximim target limit has not been reached;
    // calls AssignSpace to asign taget to a cell in the lostcity;
    // updates active_targets & target_count
    //-----------------------------------------------------------------------*/
    
    bool AssignSpecificSpace ( Target * t, int r, int c, int d );
    /**------------------------------------------------------------------------
    // takes pointer to a target and specific coordinates; assigns target to 
    // cell: private method - used for testing
    //-----------------------------------------------------------------------*/
    
    bool Hit( void );
    /**------------------------------------------------------------------------
    // prompts user for row, col and depth position that they want to hit
    // returns true if successful, false otherwise
    // updates data counts
    //-----------------------------------------------------------------------*/
    
    void Show( void ) const;
    /**------------------------------------------------------------------------
    // shows each layer of the lostcity by depth
    // returns true if successful, false otherwise
    // updates data counts
    //-----------------------------------------------------------------------*/
  
    void ShowTargetStats( void );
    /**------------------------------------------------------------------------
    // shows targets stats for all targets
    //-----------------------------------------------------------------------*/
  
    void ShowTargets( int tt ) const;
    /**------------------------------------------------------------------------
    // shows targets information of type tt 
    //-----------------------------------------------------------------------*/
 
    void ShowGameOver( void ) const;
    /**------------------------------------------------------------------------
    // displays "Game Over" banner
    //-----------------------------------------------------------------------*/
     
    int hit_value;                  // value of target hit
    bool hit_crystal;               // true if hit crystal

  private:
     //-----------------------------------------------------------Private data
    enum DIM { ROW = 10, COL = 10, DEPTH = 10, MAX_TARGET = 10, MAX_PODS = 5};
    enum USER_ROWS { A,B,C,D,E,F,G,H,I,J };
    std::string name;               // name of object
    int target_count;               // total targets
    int pod_count;                  // total pods
    int active_targets;             // total targets still active
    int active_pods;                // total pods still active
    int active_crystals;            // total crystals
    int destroyed_targets;          // total targets destroyed
    int destroyed_pods;             // total pods destroyed
    int damaged_crystals;           // number of crystals damaged
    int difficulty_level;           // user chosen value
    bool game_over;                 // true when crytal destroyed
    bool time_elapsed;              // true when time is gone
    Cell lostcity[ROW][COL][DEPTH]; // cells of the lostcity
    Target * target_list[MAX_TARGET];  // array of all targets in lostcity

    //----------------------------------------------------------Private methods
    void Init( void );
    /**------------------------------------------------------------------------
    // initializes Lostcity data
    //-----------------------------------------------------------------------*/
    
    void AssignRandomSpace ( Target * t );
    /**------------------------------------------------------------------------
    // takes pointer to a target and randomly assigns target to the number of
    // cells equal to its armor; private method - target location secret 
    // only first row and column are random - adjacent rows and col spaces 
    // are filled to keep target like one unit
    //-----------------------------------------------------------------------*/
    
  }; // end class Lostcity
  
} // namespace Atlantis

#endif // LOSTCITY_H

// eof lostcity.h
