// -------------------------------------------------------------------------\\
//          
// File:        player.h
// Author:      Bettina McCormick
//          
// Project:     Atlantis: COP 2224 VCC; Summer 1999
//          
// Created:     July 21, 1999
// Updated:     July 31, 1999
// Date due:    August 04, 1999
//             
// Description: Specification for Player class
//
//        
// Notes:       Turn on symbol PLAYER_TEST to test the component

// -------------------------------------------------------------------------//

#ifndef PLAYER_H
#define PLAYER_H

#include <ctime>
#include <iostream>
#include <iomanip>
#include <string>

#include "lostcity.h"

namespace Atlantis // project namespace
{

class Player
{
  public:
    Player();
  /**------------------------------------------------------------------------
  // public constructor: default - no arguments
  //-----------------------------------------------------------------------*/

    Player( std::string name );
  /**------------------------------------------------------------------------
  // overloaded constructor: takes reference to string for name
  //-----------------------------------------------------------------------*/
  
    ~Player();
  /**------------------------------------------------------------------------
  // destructor
  //-----------------------------------------------------------------------*/
  
    int Get_number_hits( void ) const;
  /**------------------------------------------------------------------------
  // accessor method: returns number_hits
  //-----------------------------------------------------------------------*/
    
    int Get_number_tries( void ) const;
  /**------------------------------------------------------------------------
  // accessor method: returns number_tries
  //-----------------------------------------------------------------------*/
    
    int Get_value_accrued( void ) const;
  /**------------------------------------------------------------------------
  // accessor method: returns value_accrued
  //-----------------------------------------------------------------------*/
    
    int Get_crystal_penalty( void ) const;

    std::string Get_name( void ) const;
  /**------------------------------------------------------------------------
  // accessor method: returns name
  //-----------------------------------------------------------------------*/
    
    void Set_name ( std::string n );
  /**------------------------------------------------------------------------
  // mutator method: sets name
  //-----------------------------------------------------------------------*/
    
    void Update_elapsedtime( void );
  /**------------------------------------------------------------------------
  // mutator method: comutes and updates elapsedtime
  //-----------------------------------------------------------------------*/
    
    double Get_elapsedtime( void );
  /**------------------------------------------------------------------------
  // mutator method: calls Update_elapsedtime & returns elapsedtime 
  //-----------------------------------------------------------------------*/
    
    void Cast_evils_pods( Lostcity * lostcity, int level );
  /**------------------------------------------------------------------------
  // takes a pointer to a lostcity and the difficulty level
  // calls lostcity PlaceTarget method to populate city with pods
  //-----------------------------------------------------------------------*/
    
    void AnchorCrystals( Lostcity * lostcity );
  /**------------------------------------------------------------------------
  // takes a pointer to a lost city and calls lostcity hit method
  // if hit succeeds updates number of hits players has
  //-----------------------------------------------------------------------*/
    
    bool Hit( Lostcity * lostcity );
  /**------------------------------------------------------------------------
  // takes a point to a lost city and calls lostcity hit method
  // if hit succeeds updates number of hits players has
  //-----------------------------------------------------------------------*/
    
    int Score( Lostcity * lostcity );
  /**------------------------------------------------------------------------
  // takes a point to a lostcity to access data about targets
  // computes and returns player's score
  //-----------------------------------------------------------------------*/
    
    void Show( void );
  /**------------------------------------------------------------------------
  // shows players stats & score 
  //-----------------------------------------------------------------------*/
    
  private:
    std::string   name;   // player name
    int score;            // score
    int number_tries;     // # tries to hit 
    int number_hits;      // # successful hits
    int value_accrued;    // v alue from targets hit
    int crystal_penalty;  // penalty from crystals hit
    time_t begintime;     // time startedgame
    double elapsedtime;   // time elasped
}; // end class Player

} // namespace Atlantis

#endif // eof player.h