// -------------------------------------------------------------------------\\
//          
// File:        cell.cpp
// Author:      Bettina McCormick
//          
// Project:     Atlantis: COP 2224 VCC; Summer 1999
//          
// Created:     June 18, 1999
// Updated:     July 21, 1999
// Date due:    August 04, 1999
//             
// Description: Implementaion for type Cell
//
//              A Cell is a piece of a Lostcity (Lostcity "has a" Cell)
//              A Cell "has a" Target - it contains a pointer to the base 
//              class Target.  Cell polymorphically calls Hit of the derived 
//              target object through the base class pointer 
//        
// Notes:       Turn on symbol CELL_TEST to test this component
//
// -------------------------------------------------------------------------//
    
#include "cell.h"
      
namespace Atlantis    // project namespace
{ 
    
  Cell::Cell()
  /**------------------------------------------------------------------------
  // public constructor: default - no arguments
  //-----------------------------------------------------------------------*/
  {
    hit_status = false; // assume false
    x_row = 0;              // entry pont of Lostcity
    y_col = 0;
    z_dep = 0;
    target = 0;         // contains no target
  } // end Cell()
      
      
  Cell::Cell ( const Cell & c )
  /**------------------------------------------------------------------------
  // copy constructor: takes reference to a Cell
  // private prevents passing Cell by value 
  //-----------------------------------------------------------------------*/
  {
    hit_status = false;     // assume false 
    x_row = c.Get_x();          // use accessor methods
    y_col = c.Get_y();
    z_dep = c.Get_z();
    target = c.Get_target();// get target
  } // end Cell ( const Cell & c )
    
      
  Cell::~Cell()
  /**------------------------------------------------------------------------
  // destructor
  //-----------------------------------------------------------------------*/
  {                 
    // does not cleanup the target
  } // end destructor                
  
    
  void
  Cell::Set_xyz( int x, int y, int z )
  /**------------------------------------------------------------------------
  // mutator method: sets all three coordinates of cell
  //-----------------------------------------------------------------------*/
  {
    x_row = x;  y_col = y;  z_dep = z; // row, column and depth
  } // end Set_xyz( int _x, int _y, int _z )
  
    
  inline
  int
  Cell::Get_x ( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns x_row coordinate (row)
  //-----------------------------------------------------------------------*/
  {
    return( x_row );    // row dimension
  } // end Get_x ( void ) const
  
    
  inline
  int
  Cell::Get_y ( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns y_col coordinate (row)
  //-----------------------------------------------------------------------*/
  {
    return( y_col );    // column dimension
  } // end Get_x ( void ) const
  
    
  inline                    
  int                       
  Cell::Get_z ( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns z_dep coordinate (row)
  //-----------------------------------------------------------------------*/
  {                         
    return( z_dep );    // depth dimension           
  } // end Get_z ( void ) const                        
  
    
  bool
  Cell::Set_target ( Target * t )
  /**------------------------------------------------------------------------
  // mutator method: assigns target to Cell
  // returns true if cell was emtpy and could be assigned a target;
  // returns false if cell wasd already occupied by a target
  //-----------------------------------------------------------------------*/
  {
    bool set_status = false;  // assume occupied - not able to set

    if ( target == 0 )        // if cell has no target 
    {
      target = t;             // assign target
      set_status = true;      // set return value
    }
    
    return( set_status );
  } // end Set_target ( Target * t )  
  
    
  Target *
  Cell::Get_target ( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns ponter to the target cell contains
  //-----------------------------------------------------------------------*/
  {
    return ( target );
  } // end Get_target ( void ) const
    
    
  bool
  Cell::Hit ( void )
  /**------------------------------------------------------------------------
  // polymorphically calls Hit of the derived target object through the 
  // pointer to the target
  //-----------------------------------------------------------------------*/
  {
    std::cout << "\n\tCELL  "
              << "[" << x_row + 1 << "," << y_col + 1 << "," << z_dep + 1 
              << "]" << std::endl; 
    
    bool local_hit = false;       // assume not hit
        
    if ( target )       // if cell has a target
    {
      if ( hit_status == false )  // if cell not hit
      {
        if ( target->Hit() )    // have target hit itself
        {
          hit_status = true;    // change cell status to hit
          local_hit = true;
          target->Show( hit_status );
        }
      }
      else    // otherwise this cell has been hit
      {
        local_hit = false;
        target->Show( local_hit );
      }
    }
    else      // else cell doesn't have target
    {
      ShowMiss();     // move is a miss
    }
        
    return( local_hit );
  } // end Hit ( void )
    
    
  void 
  Cell::ShowMiss ( void ) const
  /**------------------------------------------------------------------------
  // shows a "miss" - no target in cell to hit
  //-----------------------------------------------------------------------*/
  {
    std::cout << std::endl << std::endl
              << "\t*     *   *****    ***     ***   " << std::endl
              << "\t**   **     *     *   *   *   *  " << std::endl
              << "\t* * * *     *      *       *     " << std::endl
              << "\t*  *  *     *       *       *    " << std::endl
              << "\t*     *     *        *       *   " << std::endl
              << "\t*     *     *         *       *  " << std::endl
              << "\t*     *     *     *   *   *   *  " << std::endl
              << "\t*     *   *****    ***     ***   " << std::endl
              << std::endl << std::endl << std::endl << std::endl;
  } // end ShowMiss ( void ) const
    
} // namespace Atlantis
    
#ifdef CELL_TEST
  /**------------------------------------------------------------------------
  // unit test for the methods in this class
  //-----------------------------------------------------------------------*/
  
int
main (void)
{       
  int pdepth = 0;
  int cdepth = 0;
  std::cout << "Cell Unit Test\n" << std::endl;
  
  Atlantis::Pod * my_pod =          // create pod target
            Atlantis::Pod::Create( Atlantis::DESIRE  );
  Atlantis::Crystal * my_crystal =  // create crystal target
            Atlantis::Crystal::Create( Atlantis::LAUGHTER );
    
  pdepth = my_pod->Get_depth_limit();
  cdepth = my_crystal->Get_depth_limit();
  
  Atlantis::Cell layer[10][10][10];   // declare 3D array of cells
  
  for ( int i = 0; i < 10; i++ )      // fill layer 
  {
    for ( int j = 0; j < 10; j++ )
    {
      for ( int k = 0; k < 10; k++ )
      {
        layer[i][j][k].Set_xyz( i, j, k );
      }
    }
  }
  
  layer[3][1][pdepth].Set_target( my_pod );// set taget in cells
  layer[3][2][pdepth].Set_target( my_pod );
  layer[3][3][pdepth].Set_target( my_pod );
    
  layer[5][5][cdepth].Set_target( my_crystal );
      
  // fails for already popluated
  bool place_stat = layer[3][3][pdepth].Set_target( my_pod );
  
  if ( !place_stat )
  {
    std::cout << "\nFailed to place target: cell already popluated\n" 
              << std::endl;
    layer[3][3][pdepth].Get_target()->Show();
    Atlantis::Constuff::PauseScreen();
    Atlantis::Constuff::ClearScreen();
  }
  
  layer[2][1][5].Hit();           // miss
  Atlantis::Constuff::PauseScreen();
  Atlantis::Constuff::ClearScreen();

  layer[3][1][pdepth].Hit();      // hit - damage 1
  Atlantis::Constuff::PauseScreen();
  Atlantis::Constuff::ClearScreen();

  layer[3][2][pdepth].Hit();      // hit - damage 2
  Atlantis::Constuff::PauseScreen();
  Atlantis::Constuff::ClearScreen();

  layer[3][3][pdepth].Hit();      // hit - destroy
  Atlantis::Constuff::PauseScreen();
  Atlantis::Constuff::ClearScreen();

  layer[5][1][cdepth].Hit();      // miss
  Atlantis::Constuff::PauseScreen();
  Atlantis::Constuff::ClearScreen();

  layer[3][3][pdepth].Hit();      // hit already destroyed
  Atlantis::Constuff::PauseScreen();
  Atlantis::Constuff::ClearScreen();
  
  layer[5][5][cdepth].Hit();      // hit crystal - destroy
  Atlantis::Constuff::PauseScreen();
  Atlantis::Constuff::ClearScreen();

  layer[5][5][cdepth].Hit();      // hit already destroyed
  return(0);
} // end main

#endif // CELL_TEST

// eof cell.cpp

