//          
// File:        lostcity.cpp
// Author:      Bettina McCormick
//          
// Project:     Atlantis: COP 2224 VCC; Summer 1999
//          
// Created:     July 08, 1999
// Updated:     July 29, 1999
// Date due:    August 04, 1999
//             
// Description: Implementation for type Lostcity
//
//              Lostcity is a container for cells
//        
// Notes:       Turn on symbol LOSTCITY_TEST to test this component
//    
// -------------------------------------------------------------------------// 

#include <cstdlib>
#include <ctime>

#include "lostcity.h"

namespace Atlantis
{
  
  enum LostcityObjects { MAX_LOSTCITY_OBJECTS=2 };

  std::string LostcityObjectsString[MAX_LOSTCITY_OBJECTS] = 
  { "Evil Pod", "Crystal" };
  
  Lostcity::Lostcity( )
  /**------------------------------------------------------------------------
  // public constructor: default - no arguments
  // prompts user for Lostcity name
  //-----------------------------------------------------------------------*/
  {     
    Constuff::UserEntry("Lostcity Name", name, 50 );
    Init();       // initialize data
    #ifdef LOSTCITY_TEST    // for testing only
    std::cout << "Constructed Lostcity: " << name << std::endl;
    #endif
  } // end Lostcity( )
  
      
  Lostcity::Lostcity( const std::string & n )
  /**------------------------------------------------------------------------
  // overloaded constructor: takes reference to string for name
  //-----------------------------------------------------------------------*/
  {
    name = n;
    Init();
    #ifdef LOSTCITY_TEST    // for testing only
    std::cout << "Constructed Lostcity: " << name << std::endl;
    #endif
  } // end Lostcity( const std::string & n )
  
      
  Lostcity::~Lostcity()
  /**------------------------------------------------------------------------
  // destructor - when lost city is destructed, the objects in the city must 
  // also be destroyed
  //-----------------------------------------------------------------------*/
  {
    for ( int i = 0; i < MAX_TARGET; ++i )
    {       // Go through the target list and delete the objects
      if ( target_list[i] )     // check pointer
      {
        delete target_list[i];  // delete object
        target_list[i] = 0;     // set pointer to null
      }
    } // end for loop 
    #ifdef LOSTCITY_TEST      // for testing only
    std::cout << "Destructed Lostcity: " << name << std::endl;
    #endif
  } // end destructor
    
      
  void
  Lostcity::Init( void )
  /**------------------------------------------------------------------------
  // initializes Lostcity data
  //-----------------------------------------------------------------------*/
  {
    target_count = 0;               // total targets
    pod_count = 0;                  // total pods
    active_targets = 0;             // total targets still active
    active_pods = 0;                // total pods still active
    active_crystals = 0;            // total crystals
    destroyed_targets = 0;          // total targets destroyed
    destroyed_pods = 0;             // total pods destroyed
    damaged_crystals = 0;           // number of crystals damaged
    difficulty_level = MAX_PODS;    // user chosen value
    hit_value = 0;                  // value of target hit
    hit_crystal = false;               // tru if hit crysta
    game_over = false;              // true when crytal destroyed
    time_elapsed = false;              // true when time is gone
    
    for ( int c = 0; c < MAX_TARGET; ++c )
    {       // Go through the target list and initialize objects to null
      target_list[c] = 0;
    }
    
    for ( int i = 0; i < ROW; i++ )   // build lostcity
    {
      for ( int j = 0; j < COL; j++ )
      {
        for ( int k = 0; k < DEPTH; k++ )
        {
          lostcity[i][j][k].Set_xyz( i, j, k );
        }
      }
    }
    
  } // end Init( void )
  
  std::string
  Lostcity::Get_name( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns lostcity name
  //-----------------------------------------------------------------------*/
  {
    return( name );
  } // end Get_target_count( void ) const


  int 
  Lostcity::Get_target_count( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns target count
  //-----------------------------------------------------------------------*/
  {
    return( target_count );
  } // end Get_target_count( void ) const
  
    
  int 
  Lostcity::Get_active_targets( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns active targets
  //-----------------------------------------------------------------------*/
  {
    return( active_targets );
  } // end Get_active_targets( void ) const
  
    
  int 
  Lostcity::Get_destroyed_targets( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns destroyed targets
  //-----------------------------------------------------------------------*/
  {
    return( destroyed_targets );
  } // end Get_destroyed_targets( void ) const
    
    
  int 
  Lostcity::Get_destroyed_pods( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns destroyed pods
  //-----------------------------------------------------------------------*/
  {
    return( destroyed_pods );
  } // end Get_destroyed_pods( void ) const
  
  
  int 
  Lostcity::Get_damaged_crystals( void ) 
  /**------------------------------------------------------------------------
  // goes thru target list, counts how many crystals are damaged & returns
  //-----------------------------------------------------------------------*/
  {
    int local_count = 0;      // local varial to redo count each time called
      
    for ( int i = 0; i < MAX_TARGET; ++i )
    {
      if ( target_list[i] )     // go thru target list to find crystals   
      {
        if ( (target_list[i]->Get_type() == CRYSTAL ) &&
             (target_list[i]->Get_status() == DAMAGED ) )
        {                     // if crystal and damaged
            local_count++;    // count it
        }
      }
    }
    damaged_crystals = local_count;  // assign count to class variable
    
    return( damaged_crystals );     // return count
  } // end Get_damaged_crystals( void ) const
    
      
  void 
  Lostcity::Set_game_over( bool go )
  /**------------------------------------------------------------------------
  // mutator method: sets game_over
  //-----------------------------------------------------------------------*/
  {
    game_over = go;
  } // end Set_game_over( int go )

  void 
  Lostcity::Set_time_elapsed( bool te )
  /**------------------------------------------------------------------------
  // mutator method: sets time_elapsed
  //-----------------------------------------------------------------------*/
  {
    time_elapsed = te ;
  } // end Set_time_elapsed( int te )

  void
  Lostcity::Set_difficulty_level( int dl )
  /**------------------------------------------------------------------------
  // mutator method: sets difficulty_level
  //-----------------------------------------------------------------------*/
  {
    difficulty_level = dl;
  } // end Set_difficulty_level( int dl )
    
    
  int
  Lostcity::User_defines_level ( void )
  /**------------------------------------------------------------------------
  // prompts user for difficulty level 
  // level equates to number of pods and crystals that will fill the lostcity
  // returns vlaue entered by user
  //-----------------------------------------------------------------------*/
  {
    int dif_level = 0;
    std::string sepline = "--------------------------------------------\n";

    std::cout << sepline << std::endl;
    Constuff::UserEntry( "Enter the difficulty level for this game:", 
                          dif_level, 1, MAX_TARGET / 2 );
    std::cout << sepline << std::endl;
    return( dif_level );
  } // end User_defines_level ( void )
    
      
  void
  Lostcity::PlaceTarget( int object, int obj_type )
  /**------------------------------------------------------------------------
  // takes int describing the object (pod or crystal) and int defining type 
  // of pod or crystal; creates respective object & places it in the 
  // "target_list" provided that maximim target limit has not been reached;
  // calls AssignSpace to asign taget to a cell in the lostcity;
  // updates active_targets & target_count
  //-----------------------------------------------------------------------*/
  {
    if ( active_targets < MAX_TARGET )
    {       // if maximum target limit is not reached 
      switch ( object )
      {     // create indicated object
      case POD:
        {
          target_list[active_targets] = Pod::Create( obj_type );
          break;
        }
      case CRYSTAL:
        {
          target_list[active_targets] = Crystal::Create( obj_type );
          break;
        }
      default:
        {       // always provide default for safety
          std::cerr << "\nNot a valid Target Choice\n" << std::endl;
          break;
        }
      } // end switch

      if ( target_list[active_targets] )// assign randomly
      {                                 // update data counts
        AssignRandomSpace ( target_list[active_targets] );    
        target_list[active_targets]->Set_inLostcity( true ); 
        if (target_list[active_targets]->Get_type() == POD)
        {
          active_pods++;      // pods
          pod_count++;
        }
        else 
        {
          active_crystals++;  // crystals
        }
        target_count++;
        active_targets++;
      }
    } // end if maximum target limit is not reached 
  } // end PlaceTarget( int object, int obj_type )
  
    
  void
  Lostcity::AssignRandomSpace ( Target * t )
  /**------------------------------------------------------------------------
  // takes pointer to a target and randomly assigns target to the number of
  // cells equal to its armor; 
  // only first row and cloumn are random - adjacent rows and col spaces 
  // are filled to keep target like one unit
  //-----------------------------------------------------------------------*/
  {
    bool conflict = true; // true if try to assign cell already filled
    int x, y = 0;  // for incrementation
    int r = 0;  // row  
    int c = 0;  // column
    int d = t->Get_depth_limit();
    
    while ( conflict == true ) // keep trying untils not confilcts
    {
      r = Atlantis::Constuff::GetRandNum( ROW );  // assign random row
      c = Atlantis::Constuff::GetRandNum( COL );  // assign random column
    
      if (t) // check pointer
      {
        if ( r > c) 
        {// choose max of r or c to determine direction of target placement
          if ( (ROW - r) >= t->Get_armor() ) // expand target along row
          {     // if enough space  - increment assignment process
            for (x = r - 1; x < ( (r-1) + t->Get_armor() ); x++)
            {
              if ( lostcity[x][c - 1][d - 1].Get_target() == 0 )
              {
                lostcity[x][c - 1][d - 1].Set_target( t );
              }
              else
              {
                conflict = true; // cell already populated 
                break; // start over
              }
              conflict = false;
            }
          }
          else  // else decrement assignment process
          {
            for ( x = r - 1; x >= ( r - t->Get_armor() ); x--)
            {
              if ( lostcity[x][c - 1][d - 1].Get_target() == 0 )
              {
                lostcity[x][c - 1][d - 1].Set_target( t );
              }
              else
              {
                conflict = true; // cell already populated 
                break; // start over
              }
              conflict = false;
            }
          }
        }
        else // expand target along column
        {
          if ( (COL - c) >= t->Get_armor() ) 
          {     // if enough space  - increment assignment process
           for ( y = c - 1; y < ( (c-1) + t->Get_armor() ); y++)
            {
              if ( lostcity[r - 1][y][d - 1].Get_target() == 0 )
              {
                lostcity[r - 1][y][d - 1].Set_target( t );
              }
              else
              {
                conflict = true; // cell already populated 
                break; // start over
              }
              conflict = false;
            }
          }
          else  // else decrement assignment process
          {
            for ( y = c - 1; y >= ( c - t->Get_armor() ); y--)
            {
              if ( lostcity[r - 1][y][d - 1].Get_target() == 0 )
              {
                lostcity[r - 1][y][d - 1].Set_target( t );
              }
              else
              {
                conflict = true; // cell already populated 
                break; // start over
              }
              conflict = false;
            }
          }
        }
      } // end if (t)
    } // end while
  } // end AssignSpace ( Target * t )
    
  
  bool 
  Lostcity::AssignSpecificSpace ( Target * t, int r, int c, int d )
  /**------------------------------------------------------------------------
  // takes pointer to a target and specific coordinates; assigns target to 
  // cell: private method - used for testing
  //-----------------------------------------------------------------------*/
  {
    bool assigned = true;

    if ( lostcity[r-1][c-1][d-1].Get_target() == 0 )  // check for target
    {
        lostcity[r-1][c-1][d-1].Set_target(t); // if none, assign the sapce
    }
    else
    {       // else space occupied
      assigned = false;
    }
      return ( assigned );
  } // end AssignSpecificSpace ( Target * t, int r, int c, int d )
    
      
  bool
  Lostcity::Hit ( void )
  /**------------------------------------------------------------------------
  // prompts user for row, col and depth position that they want to hit
  // returns true if successful, false otherwise
  // updates data counts
  //-----------------------------------------------------------------------*/
  {
    int r = 0;      // temp variable for user input
    int c = 0;
    int d = 0;
    bool status = false;    // hit assume false
    
    std::cout << "\nHit a spot in " << Get_name();      // prompt user
    Atlantis::Constuff::UserEntry ( "Row Position", r, 1, ROW );
    Constuff::UserEntry ( "Column Position", c, 1, COL );
    Constuff::UserEntry ( "Depth Position", d, 1, DEPTH );
    
    status = lostcity[r-1][c-1][d-1].Hit();           // hit cell
      
    if ( status == true )             // if hit, update
    {
      hit_value = lostcity[r-1][c-1][d-1].Get_target()->Get_value();
      if ( lostcity[r-1][c-1][d-1].Get_target()->Get_type() == CRYSTAL )
      {
        hit_crystal = true;
        damaged_crystals++;     // record damage
      }
      else
      {
        hit_crystal = false;
      }
      if (lostcity[r-1][c-1][d-1].Get_target()->Get_status() == DESTROYED)
      {
        if ( lostcity[r-1][c-1][d-1].Get_target()->Get_type() == CRYSTAL )
        {
          Set_game_over( true );         // crystals
          Constuff::PauseScreen();
          ShowGameOver();
          active_crystals--;
        }
        else
        {
          destroyed_pods++;             // pods
          active_pods--;
        }
        destroyed_targets++;            // totals 
        active_targets--;
      }
    }
    return ( status );
  } // end Hit ( void )
  
    
  void 
  Lostcity::Show ( void ) const
  /**------------------------------------------------------------------------
  // shows each layer of the lostcity by depth
  // returns true if successful, false otherwise
  // updates data counts
  //-----------------------------------------------------------------------*/  
  {
    int i = 0;      // for incrementation
    int j = 0;
    int d = 0;
    std::string tt = "PC"; // string to access chars

    for ( d = 0; d < DEPTH; ++d )
    {         // for each depth layer
      Constuff::ClearScreen();
      std::cout << "Lostcity Depth: " << d+1 << std::endl;
      std::cout << "------------------------------------------" << std::endl;
      std::cout << "    1   2   3   4   5   6   7   8   9   10" << std::endl;
      for ( i = 0; i < ROW; ++i )
      {
        std::cout << std::endl << i+1;
        if ( i < 9 )
        {             // show rows 
          std::cout << "   ";
        }
        else
        {             // space
          std::cout << "  ";
        }
        for ( j = 0; j < COL; ++j )
        {
          if ( lostcity[i][j][d].Get_target() )
          {           // show columns
            std::cout << tt[ lostcity[i][j][d].Get_target()->Get_type() ]
                      << lostcity[i][j][d].Get_target()->Get_armor() 
                      << "  ";
          }
          else
          {           // show empty spots
            std::cout << "-   ";
          }
        }
        std::cout << std::endl;
      }
      Atlantis::Constuff::PauseScreen(); // pause at end of each depth layer
    }
  } // end Show ( void )
  
    
  void 
  Lostcity::ShowTargetStats( void ) 
  /**------------------------------------------------------------------------
  // shows targets stats for all targets
  //-----------------------------------------------------------------------*/
  {
    std::string sepline = "\n--------------------------------------------\n";
                                // display target stats
    std::cout << "\n\t\tLostcity Target Status" << sepline 
              << "Pod Count        : " << pod_count << "\n" 
              << "Active Pods      : " << active_pods << "\n" 
              << "Destroyed Pods   : " << destroyed_pods << "\n" 
              << "Damaged Crystals : " << Get_damaged_crystals() 
              << sepline << std::endl;
  } // end ShowsTargetStats( void )
    
    
  void 
  Lostcity::ShowTargets( int tt ) const
  /**------------------------------------------------------------------------
  // shows targets information of type tt 
  //-----------------------------------------------------------------------*/
  {
    std::string sepline = "\n--------------------------------------------\n";
    
    if ( tt == POD )        // type is pod
    {
      std::cout << "\n\t\tLostcity Pods " << sepline << std::endl;
    }
    else                    // type is crystal
    {
      std::cout << "\nLostcity Crystals " << sepline << std::endl;
    }

    for ( int i = 0; i < MAX_TARGET; ++i )
    {                 // cycle through target list
      if ( target_list[i] )         // check pointer
      {
        if (target_list[i]->Get_type() == tt )
        {
          target_list[i]->Show();
          std::cout << sepline << std::endl;
          Constuff::PauseScreen();
        }
      }
      
    }
  } // end ShowTargets( int tt ) const
    
    
  void 
  Lostcity::ShowGameOver( void ) const
  /**------------------------------------------------------------------------
  // displays "Game Over" banner
  //-----------------------------------------------------------------------*/
  {
    std::cout << std::endl << std::endl
              << "\t ***     **   *     *  ****  " << std::endl
              << "\t*       *  *  **   **  *     " << std::endl
              << "\t* ***   ****  * * * *  ***   " << std::endl
              << "\t*   *   *  *  *  *  *  *     " << std::endl
              << "\t ***    *  *  *     *  ****  " << std::endl
              << std::endl
              << "\t                              ***   " << std::endl
              << "\t                             *   *  " << std::endl
              << "\t   **    *   *  ****  ***    *   *  " << std::endl
              << "\t  *  *   *   *  *     *  *    * *   " << std::endl
              << "\t  *  *   *   *  ***   ***      *    " << std::endl
              << "\t  *  *    * *   *     *  *          " << std::endl
              << "\t   **      *    ****  *   *    *    " << std::endl
              << std::endl
              << "You have destroyed a Crystal!!" 
              << std::endl << std::endl;
  } // end ShowGameOver( void ) const
  
  
} // namespace Atlantis

#ifdef LOSTCITY_TEST
  /**------------------------------------------------------------------------
  // unit test for the methods in this class
  //-----------------------------------------------------------------------*/

int
main ( void )
{
  bool space_filled = false;      // assume false
  int i = 0;                      // basic counter
  int p_d = 0;                    // pod depth for testing
  int c_d = 0;                    // crystal depth for testing
      
  std::cout << "Lostcity Unit Test\n" << std::endl;
  // create generic pod and crystal to test
  Atlantis::Target * gen_pod = Atlantis::Pod::Create( Atlantis::IDLENESS );
  Atlantis::Target * gen_crystal = 
            Atlantis::Crystal::Create( Atlantis::LAUGHTER );
      
  // get depths
  p_d = gen_pod->Get_depth_limit();
  c_d = gen_crystal->Get_depth_limit(); 
    
  // create lostcity
  Atlantis::Lostcity city_atlantis("Atlantis");
    
  // randomly place targets in lostcity
  // pods
  city_atlantis.PlaceTarget( Atlantis::POD, Atlantis::IDLENESS);
  city_atlantis.PlaceTarget( Atlantis::POD, Atlantis::DESIRE);
  city_atlantis.PlaceTarget( Atlantis::POD, Atlantis::ENVY);
  city_atlantis.PlaceTarget( Atlantis::POD, Atlantis::SELFISHNESS);
  city_atlantis.PlaceTarget( Atlantis::POD, Atlantis::GREED);
  // crystals
  city_atlantis.PlaceTarget( Atlantis::CRYSTAL, Atlantis::LAUGHTER);
  city_atlantis.PlaceTarget( Atlantis::CRYSTAL, Atlantis::FORGIVENESS);
  city_atlantis.PlaceTarget( Atlantis::CRYSTAL, Atlantis::PEACE);
  city_atlantis.PlaceTarget( Atlantis::CRYSTAL, Atlantis::JOY);
  city_atlantis.PlaceTarget( Atlantis::CRYSTAL, Atlantis::LOVE);
  
  // assign specific space
  while ( space_filled == false )
  {
    i = 2;
    // show pod location
    std::cout << "Assigning pod to : "
              << "[" << 1 << "][" << i << "][" << p_d << "] \n"
              << std::endl;
    space_filled  = city_atlantis.AssignSpecificSpace(gen_pod, 1, i, p_d);
    i++;
  } // end while

  city_atlantis.Hit(); // prompt to hit space
  Atlantis::Constuff::PauseScreen();
  Atlantis::Constuff::ClearScreen();
    
  space_filled = false;
  // assign specific space
  while ( space_filled == false )
  {
    i = 1;
    // show pod location
    std::cout << "Assigning crystal to : "
              << "[" << i << "][" << 5 << "][" << c_d << "] \n"
              << std::endl;
    space_filled  = city_atlantis.AssignSpecificSpace(gen_crystal, i, 5, c_d);
    i++;
  } // end while

  city_atlantis.Hit(); // prompt to hit space
  Atlantis::Constuff::PauseScreen();
  Atlantis::Constuff::ClearScreen();  
    
  // try random hit
  std::cout << "Try a random hit...\n";
  city_atlantis.Hit(); // prompt to hit space
  Atlantis::Constuff::PauseScreen();
  Atlantis::Constuff::ClearScreen();  

  city_atlantis.Show();         // reveal target locations
  Atlantis::Constuff::PauseScreen();
  Atlantis::Constuff::ClearScreen();  

  city_atlantis.ShowTargetStats();                // shows stats
  Atlantis::Constuff::PauseScreen();  
  Atlantis::Constuff::ClearScreen();

  city_atlantis.ShowTargets( Atlantis::POD );     // shows pods
  Atlantis::Constuff::ClearScreen();

  city_atlantis.ShowTargets( Atlantis::CRYSTAL ); // shows crystals 
  
  return(0);
} // end main

#endif // LOSTCITY_TEST

// eof lostcity.cpp

