// -------------------------------------------------------------------------\\
//          
// File:        player.cpp
// Author:      Bettina McCormick
//          
// Project:     Atlantis: COP 2224 VCC; Summer 1999
//          
// Created:     July 21, 1999
// Updated:     July 31, 1999
// Date due:    August 04, 1999
//             
// Description: Implementation for Player class
//
//        
// Notes:       Turn on symbol PLAYER_TEST to test the component

// -------------------------------------------------------------------------//

#include "player.h"

namespace Atlantis // project namespace
{
  
  Player::Player()
  /**------------------------------------------------------------------------
  // public constructor: default - no arguments
  //-----------------------------------------------------------------------*/
  {
    begintime = time(0);    // assign time
    score = 0;              // initialized data
    number_tries = 0;
    number_hits = 0;
    value_accrued = 0;
    crystal_penalty = 0;
    Constuff::UserEntry( "Player Name", name, 50 ); // default name
    #ifdef PLAYER_TEST    // for testing only
    std::cout << "Constructed Player: " << name << std::endl;
    #endif
  } // end no argument constructor
  
  
  Player::Player( std::string n )
  /**------------------------------------------------------------------------
  // overloaded constructor: takes reference to string for name
  //-----------------------------------------------------------------------*/
  {
    name = n;               // assign value
    begintime = time(0);
    score = 0;              // initialize data
    number_tries = 0;
    number_hits = 0;
    value_accrued = 0;
    crystal_penalty = 0;
    #ifdef PLAYER_TEST    // for testing only
    std::cout << "Constructed Player: " << name << std::endl;
    #endif
  } // end Player( std::string n )
    
  
  Player::~Player()
  /**------------------------------------------------------------------------
  // destructor
  //-----------------------------------------------------------------------*/
  {
    // no dynamic memory to clean up
  } // end destructor
  
    
    
  int 
  Player::Get_number_hits( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns number_hits
  //-----------------------------------------------------------------------*/
  {
    return( number_hits );
  }// end Get_number_hits( void ) const
  
    
  int 
  Player::Get_number_tries( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns number_tries
  //-----------------------------------------------------------------------*/
  {
    return( number_tries );
  } // end Get_number_tries( void ) const
  
    
  int 
  Player::Get_value_accrued( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns value_accrued
  //-----------------------------------------------------------------------*/
  {
    return( value_accrued );
  } // end Get_value_accrued( void ) const 
    
    
  int 
  Player::Get_crystal_penalty( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns crystal_penalty
  //-----------------------------------------------------------------------*/
  {
    return( crystal_penalty );
  } // end Get_crystal_penalty( void ) const
    
    
  std::string 
  Player::Get_name( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns name
  //-----------------------------------------------------------------------*/
  {
    return (name);
  } // end Get_name( void ) const
  
    
  void 
  Player::Set_name ( std::string n )
  /**------------------------------------------------------------------------
  // mutator method: sets name
  //-----------------------------------------------------------------------*/
  {
    name = n;
  } // end Set_name ( std::string n )
  
  
  void 
  Player::Update_elapsedtime( void ) 
  /**------------------------------------------------------------------------
  // mutator method: comutes and updates elapsedtime
  //-----------------------------------------------------------------------*/
  {     
    elapsedtime = ( static_cast<double>( ( time(0)-begintime ) ) / 60.0 );
  } // end Update_elapsedtime( void ) 
  
      
  double 
  Player::Get_elapsedtime( void )
  /**------------------------------------------------------------------------
  // mutator method: calls Update_elapsedtime & returns elapsedtime 
  //-----------------------------------------------------------------------*/
  {
    Update_elapsedtime(); 
    return ( elapsedtime );
  } // end Get_elapsedtime( void ) const
    
    
  void 
  Player::Cast_evils_pods( Lostcity * lostcity, int level )
  /**------------------------------------------------------------------------
  // takes a pointer to a lostcity and the difficulty level
  // calls lostcity PlaceTarget method to populate city with pods
  //-----------------------------------------------------------------------*/
  {
    for ( int i = 0; i < level; i++)
    {
      lostcity->PlaceTarget(Atlantis::POD, Atlantis::IDLENESS + i);
    }
  } // end Cast_evils_pods( Lostcity * lostcity, int level )

  void 
  Player::AnchorCrystals( Lostcity * lostcity )
  /**------------------------------------------------------------------------
  // takes a pointer to a lost city and calls lostcity hit method
  // if hit succeeds updates number of hits players has
  //-----------------------------------------------------------------------*/
  {
    for ( int i = 0; i < 5;  i++)
    {
      lostcity->PlaceTarget(Atlantis::CRYSTAL, Atlantis::LAUGHTER + i);
    }
  } // end AnchorCrystals( Lostcity * lostcity )

    
  bool 
  Player::Hit( Lostcity * lostcity )
  /**------------------------------------------------------------------------
  // takes a point to a lost city and calls lostcity hit method
  // if hit succeeds updates number of hits players has
  //-----------------------------------------------------------------------*/
  {
    bool status = lostcity->Hit(); // try hit
    int targetval = 0;

    number_tries++;                   // increment

    targetval = lostcity->hit_value;  // get value of target hit
    value_accrued += targetval;        // calc value acrued

    if ( lostcity->hit_crystal == true )
    {
      crystal_penalty += targetval;  // add penalty if crystal
    }

    if (status == true) // if hit increment # hits
    {
      number_hits++;    
    }
    
    score = Score( lostcity );    // update score
    return( status );
  } // end Hit( Lostcity * lostcity )
  
      
  int 
  Player::Score( Lostcity * lostcity )
  /**------------------------------------------------------------------------
  // takes a point to a lostcity to access data about targets
  // computes and returns player's score
  //-----------------------------------------------------------------------*/
  {
    int my_score = 0;
    my_score =(500 + (lostcity->Get_destroyed_targets() * 10)// calculate score
            + ( (2 * number_hits ) - number_tries)   // different data
            + ( value_accrued * 2 ) - crystal_penalty ); // from hit
      return( my_score ); // return
  } // end Score( Lostcity * lostcity )
  
  void 
  Player::Show( void )
  {
    double h_m_ratio = 0.0;   // hit miss ration

    if ( number_tries > 0)
    {
       h_m_ratio =(static_cast<double>(number_hits) // calc h_m ration
      / static_cast<double>(number_tries));
    }

    std::cout << std::endl;     // players stats
    std::cout.setf(std::ios::fixed);
    std::cout << std::setprecision(2);
    std::cout << "Player : " << name << std::endl;
    std::cout << "Elapsed Time: " << Get_elapsedtime() // time spent
              << " minutes " << std::endl;
    std::cout << "Score  : " << score << std::endl; // score
    std::cout << "Hit/Miss: " << h_m_ratio << std::endl; // ration
    std::cout.unsetf(std::ios::fixed);
  } // end Show( void ) 
  
}  // namespace Atlantis


#ifdef PLAYER_TEST
  /**------------------------------------------------------------------------
  // unit test for the methods in this class
  //-----------------------------------------------------------------------*/
    
int
main ( void )
{
  int level = 0;    // difficulty
    
  Atlantis::Lostcity city_atlantis("Atlantis"); // create city

  Atlantis::Player * villian = new Atlantis::Player( "Gibrato" );
  Atlantis::Player * hero = new Atlantis::Player( "Aryam" );  
          // create players
  
  level = city_atlantis.User_defines_level(); // get level

  hero->AnchorCrystals( &city_atlantis ); // fill city 
  villian->Cast_evils_pods( &city_atlantis, level );// cast pods

  hero->Hit( &city_atlantis );  // try some hits
  hero->Hit( &city_atlantis );
  hero->Hit( &city_atlantis );

  hero->Show();   // show player

  return(0);
}
#endif // PLAYER_TEST

// eof player.cpp


