// -------------------------------------------------------------------------\\
//          
// File:        pod.cpp
// Author:      Bettina McCormick
//          
// Project:     Atlantis: COP 2224 VCC; Summer 1999
//          
// Created:     June 30, 1999
// Updated:     July 25, 1999
// Date due:    August 04, 1999
//             
// Description: Implementation for Pod class
//
//              Inherits from Target - Pod "is a" Target
//        
// Notes:       Turn on symbol POD_TEST to test the component
//      
// -------------------------------------------------------------------------//
      
#include "pod.h"      // specification
      
namespace Atlantis    // project namespace
{  
  
  static const std::string PodTypeString[] = 
  {                                 
      "Idleness",
      "Desire",
      "Envy",
      "Selfishness",
      "Greed"
  }; // const array of strings for use in display
    
    
  Pod *
  Pod::Create( int pod_type )
  /**------------------------------------------------------------------------
  // no public constructor: this calls private constructor to create instance
  // of pod and returns Pod *
  //-----------------------------------------------------------------------*/
  {
    std::string podname;  // temp values mirroring the data of a target
    int d = 0;            // to be passed to target constructor
    int a = 0;
    int v = 0;
    int t = POD;

    switch ( pod_type ) 
    {                     // set values according to the pod type
    case IDLENESS:
      podname = PodTypeString[IDLENESS];// least threatening pod
      d = Constuff::GetRandNum(MAX_DEPTH);  // get random depth level
      a = MIN_ARMOR;                        // least armor protection
      v = a * 100;                  // value is multiple of armor
      break;
    case DESIRE:
      podname = PodTypeString[DESIRE]; // pods increase in size
      d = Constuff::GetRandNum(MAX_DEPTH);
      a = MIN_ARMOR + 1;
      v = a * 50;  // multiple of armor varies - either 100 or 50
      break;
    case ENVY:
      podname = PodTypeString[ENVY];
      d = Constuff::GetRandNum(MAX_DEPTH);  // max depth possible varies
      a = MIN_ARMOR + 2;
      v = a * 100;  // alternate multiple value
      break;
    case SELFISHNESS:
      podname = PodTypeString[SELFISHNESS];
      d = Constuff::GetRandNum(MAX_DEPTH);
      a = MIN_ARMOR + 3;
      v = a * 50;
      break;
    case GREED:
      podname = PodTypeString[GREED];   // most threatening Pod
      d = Constuff::GetRandNum(MAX_DEPTH);     // larget maximum depth
      a = MIN_ARMOR + 4;                // most armor protection
      v = a * 100;                      // largest value
      break;  
    default:
      podname = PodTypeString[GREED];
      d = Constuff::GetRandNum(MAX_DEPTH);     // default is worst Pod
      a = MIN_ARMOR + 4;                // most armor protection
      v = a * 100;
      break;
    };
    
    Pod * newPod = new Pod ( podname, a, d, v, t ); // call constructor
    return( newPod );         // return Pod
  } // end Create( int pod_type )
    
    
  Pod::Pod( std::string name, int armor, int maxdepth, int val,int t_type ):
  Target( name, armor, maxdepth, val, t_type )
  /**------------------------------------------------------------------------
  // private constructor calls virtual constructor of base class
  //-----------------------------------------------------------------------*/
  {
    #ifdef POD_TEST
    std::cout << "Constructed Pod " << name << std::endl;
    #endif
  } // end Pod( std::string name, int a, int d, int t )
    
    
  Pod::~Pod()
  /**------------------------------------------------------------------------
  // destructor
  //-----------------------------------------------------------------------*/
  {
    // no dynamic memory created in constructor 
    // this will be called when pointer to Pod is deleted
    #ifdef POD_TEST // only for testing
    std::cout << "Client code has successfully Destructed Pod " 
              << name << std::endl;
    #endif
  } // end destructor
  
  void 
  Pod::Abstract ( void ) 
  /**------------------------------------------------------------------------
  // fullfills virtual bass class method
  //-----------------------------------------------------------------------*/
  {       
    /* no body */
  }     

    
  bool
  Pod::Hit ( void )
  /**------------------------------------------------------------------------
  // returns true and hits Pod if not already destroyed, false otherwise, 
  //-----------------------------------------------------------------------*/
  {
    bool hit_status = Target::Hit(); // true if taget is hitable
  
    return( hit_status );
  } // end Hit ( void )
    
      
  void
  Pod::Show ( void ) const
  /**------------------------------------------------------------------------
  // calls base class method: shows pods name, armor level, hits & status
  //-----------------------------------------------------------------------*/
  {
    Target::Show();
  } // end Show ( void ) const
    
    
  void
  Pod::Show ( bool hitflag ) const
  /**------------------------------------------------------------------------
  // overloaded show method: hitflag is true if pod is hit, false if missed
  //-----------------------------------------------------------------------*/
  {   
    Target::Show( hitflag );    // calls base class method
  } // end Show ( bool hitflag ) const
  
} // namespace Atlantis
    
    
#ifdef POD_TEST
  /**------------------------------------------------------------------------
  // unit test for the methods in this class
  //-----------------------------------------------------------------------*/
  
int
main ( void )
{
  std::cout << "Pod Unit Test\n" << std::endl;
      
  bool hit = true;
  int type = 0;
  // declare pointer to pod
  Atlantis::Pod * my_pod = Atlantis::Pod::Create( Atlantis::ENVY );
  std::cout << std::endl;

  type = my_pod->Get_type();       // show target type
  std::cout << "Target is " << Atlantis::TargetTypeString [ type ] 
            << "\nValue of target is " << my_pod->Get_value() << std::endl;
  std::cout << std::endl;
    
  hit = my_pod->Hit();                           // hit and show
  my_pod->Show( hit );                           // 1 damage
  Atlantis::Constuff::PauseScreen();  
  Atlantis::Constuff::ClearScreen(); 
  hit = my_pod->Hit();                           // hit and show
  my_pod->Show( hit );                           // 2 damage
  Atlantis::Constuff::PauseScreen();  
  Atlantis::Constuff::ClearScreen(); 
  hit = my_pod->Hit();                           // hit and show
  my_pod->Show( hit );                           // 3 damage
  Atlantis::Constuff::PauseScreen();  
  Atlantis::Constuff::ClearScreen(); 
  hit = my_pod->Hit();                           // hit and show
  my_pod->Show( hit );                           // destroyed  
  Atlantis::Constuff::PauseScreen();  
  Atlantis::Constuff::ClearScreen(); 
  hit = my_pod->Hit();                           // hit and show
  my_pod->Show( hit );                           // already destroyed  
  Atlantis::Constuff::PauseScreen();

  std::cout << "\nResetting target:" << std::endl;
  my_pod->Reset();
  my_pod->Show();

  if ( my_pod )    // check pointer
  {
    delete my_pod; // delete pod
    my_pod = 0;    // set pointer null
  }
  
  return (0);
} // end main
    
#endif // POD_TEST

// eof pod.cpp

