// -------------------------------------------------------------------------\\
//          
// File:        target.cpp
// Author:      Bettina McCormick
//          
// Project:     Atlantis: COP 2224 VCC; Summer 1999
//          
// Created:     June 05, 1999
// Updated:     July 25, 1999
// Date due:    August 04, 1999
//             
// Description: Implementation for Target base class
//
//              Base class - contains virtual methods that must be implemented 
//              by subclasses that inherit from it
//        
// Notes:       Turn on symbol TARGET_TEST to test the component

// -------------------------------------------------------------------------//
    
#include "target.h"
    
namespace Atlantis // project namespace
{
  
  Target::Target( std::string n, int a, int d, int v, int t )
  /**------------------------------------------------------------------------
  // public constructor: takes string name, armor level, depth limit & type
  //-----------------------------------------------------------------------*/
  {
    name = n;                     // set initial values
    armor  = a;
    hits   = 0;
    depth_limit = d;
    value = v;
    target_type = t;
    status = NO_DAMAGE;

    inLostcity = false;           // not placed in lostcity yet
    
    srand( time ( NULL ) );       // seed rand with time

    #ifdef TARGET_TEST             // for test only
    std::cout << "Constructed Target: " << name << std::endl;
    Show();
    #endif
  } // end Target( std::string n, int a, int d )
    
    
  Target::Target( const Target & t )
  /**------------------------------------------------------------------------
  // copy constructor: takes reference to a Target
  // private prevents passing target by value 
  //-----------------------------------------------------------------------*/
  {
    #ifdef TARGET_TEST
    std::cout << "Constructed Target with private constructor: " << name 
              << std::endl;
    Show();
    #endif
  } // end Target( const Target & t )
    
    
  Target::~Target()
  /**------------------------------------------------------------------------
  // destructor: virtual
  //-----------------------------------------------------------------------*/
  {
    // no dynamic memory created in constructor
    #ifdef TARGET_TEST
    std::cout << "Destructed Target: " << name << std::endl;
    Show();
    #endif
  } // end ~Target()
    
    
  TargetStatus
  Target::Get_status ( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns status of target
  //-----------------------------------------------------------------------*/
  {
    return ( status );
  } // end Get_status ( void ) const
    
    
  int 
  Target::Get_depth_limit ( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns depth limit of target
  //-----------------------------------------------------------------------*/
  {
    return( depth_limit );
  } // end Get_depth_limit ( void ) const
    
    
  int 
  Target::Get_armor ( void )  const
  /**------------------------------------------------------------------------
  // accessor method: returns armor level of target
  //-----------------------------------------------------------------------*/
  { 
    return( armor ); 
  } // end Get_armor ( void )  const
    
    
  int 
  Target::Get_value( void ) const
  /**------------------------------------------------------------------------
  // accessor method: returns value of pod
  //-----------------------------------------------------------------------*/
  {
    return ( value );
  } // end Get_value( int pod_type ) const
    
    
  int 
  Target::Get_type ( void )  const
  /**------------------------------------------------------------------------
  // accessor method: returns type of target
  //-----------------------------------------------------------------------*/
  { 
    return( target_type ); 
  } // end Get_type ( void )  const
    
      
  void 
  Target::Set_inLostcity( bool ilc ) 
  /**------------------------------------------------------------------------
  // mutator method: sets bool data inLostcity, true or false
  //-----------------------------------------------------------------------*/
  {
    inLostcity = ilc;
  } // end Set_inLostcity( bool ilc )  
    
    
  void
  Target::Reset ( void )
  /**------------------------------------------------------------------------
  // mutator method: resets data to new target status
  //-----------------------------------------------------------------------*/
  {
    hits = 0;
    status = NO_DAMAGE;
  } // end Reset ( void )
      
      
  bool
  Target::Hit ( void )
  /**------------------------------------------------------------------------
  // hits target; updates status; returns true if target is alive and hitable
  //-----------------------------------------------------------------------*/
  {
    bool hit_state = false; // assume target has been destroyed, (no hit)
    
    if ( hits < armor )     // if hits less, then target not yet destroyed
    {
      hits++;               // record hit
      hit_state = true;     // indicate hit succeeds
      if ( hits == armor )
      {
        status = DESTROYED; // if fatal, change status
      }
      else
      {
        status = DAMAGED;   // else just damaged, change status
      }
    }
    else  // otherwise, as assumed - target was already destroyed
    {
      status = DESTROYED;
    }
         // true if hit succeeded, false if already destroyed
    return( hit_state ); 
  } // end Hit ( void )
    
    
  void 
  Target::Show ( void ) const
  /**------------------------------------------------------------------------
  // shows the targets name, armor level, hits & status
  //-----------------------------------------------------------------------*/
  {
    std::cout << "\t" << TargetTypeString[Get_type()] << " " 
              << name << std::endl
              << "\tArmor    : " << armor << std::endl
              << "\tHits     : " << hits << std::endl
              << "\tStatus   : " << StatusString[ static_cast<int>(status) ] 
              << std::endl;
  } // end Show ( void ) const
  
  
  void 
  Target::Show ( bool is_hit ) const
  /**------------------------------------------------------------------------
  // overloaded show method: is_hit is true if pod is hit, false if missed
  //-----------------------------------------------------------------------*/
  {
    
    if ( ( is_hit ) && ( status != DESTROYED ) )
    {           // if target is hit
      std::cout << std::endl << std::endl // fun banner display
                << "\t*     *    *****   *******" << std::endl
                << "\t*     *      *        *   " << std::endl
                << "\t*     *      *        *   " << std::endl
                << "\t*******      *        *   " << std::endl
                << "\t*     *      *        *   " << std::endl
                << "\t*     *      *        *   " << std::endl
                << "\t*     *      *        *   " << std::endl
                << "\t*     *    *****      *   " << std::endl
                << std::endl << std::endl << std::endl;
      Show(); // then show taget that was hit
      std::cout <<std::endl;
    }
    else if (( is_hit ) && ( status == DESTROYED ))
    {
      std::cout << std::endl << std::endl // fun banner display
                << "\t***   ****   ***  *****  ***     **   *   *  ****  *** "
                << std::endl
                << "\t*  *  *     *       *    *  *   *  *   * *   *     *  *"
                << std::endl
                << "\t*  *  ***    **     *    ***    *  *    *    ***   *  *"
                << std::endl
                << "\t* *   *        *    *    *  *   *  *    *    *     * * "
                << std::endl
                << "\t**    ****  ***     *    *   *   **     *    ****  **  "
                << std::endl << std::endl << std::endl
                << "\tYou have destroyed a " << TargetTypeString[target_type]
                << "!!" // target destroyed
                << std::endl << std::endl;
      Show(); // show target
    }
    else      // else target was already destroyed
    {         // move is wasted
      std::cout << std::endl << std::endl
                << "\t*     *    **    ***  *****  ****  ***  " << std::endl
                << "\t*  *  *   *  *  *       *    *     *  * " << std::endl
                << "\t* * * *   ****   **     *    ***   *  * " << std::endl
                << "\t**   **   *  *     *    *    *     * *  " << std::endl
                << "\t*     *   *  *  ***     *    ****  **   " << std::endl
                << std::endl
                << "\t     *     *    **    *   *  ****       " << std::endl
                << "\t     **   **   *  *   *   *  *          " << std::endl
                << "\t     * * * *   *  *   *   *  ***        " << std::endl
                << "\t     *  *  *   *  *    * *   *          " << std::endl
                << "\t     *     *    **      *    ****       " << std::endl
                << std::endl << std::endl << std::endl
                << "\tThis " << TargetTypeString[target_type]
                << " has already been destroyed!"
                << std::endl << std::endl << std::endl;
      Show();
      std::cout <<std::endl;
    }
  } // end Show ( bool is_hit ) const
      
} // namespace Atlantis
      
      
#ifdef TARGET_TEST
  /**------------------------------------------------------------------------
  // unit test for the methods in this class
  //-----------------------------------------------------------------------*/
      
int
main ( void )
{
  std::cout << "Target Unit Test\n" << std::endl;
    
  bool hit = true;
  Atlantis::Target t( "Generic Target", 3 ); //  create basic target
  
  hit = t.Hit();                             // hit and show
  t.Show( hit );                             // 1 damage
  Atlantis::Constuff::PauseScreen();
  hit = t.Hit();                             // hit and show
  t.Show( hit );                             // 2 damage
  Atlantis::Constuff::PauseScreen();
  hit = t.Hit();                             // hit and show
  t.Show( hit );                             // 3 destroyed
  Atlantis::Constuff::PauseScreen();
  hit = t.Hit();                             // hit and show
  t.Show( hit );                                 // already destroyed
  Atlantis::Constuff::PauseScreen();
  return (0);
  
} // end main
    
#endif // TARGET_TEST
    
// eof target.cpp

