// ***************************************************************************
//                         
// File:        ball.cpp
// Author:      Bettina McCormick
//                               
// Project:     Midterm: COP 2224 VCC; Summer 1999
// Template:    Created by Professor Richard Mudge
//
// Created:     June 12, 1999
// Updated:     June 15, 1999
// Date due:    June 28, 1999
//                         
// Description: Implementation for the ball object that is used with the lotto 
//              object in a simulation of a lotto machine.
//                               
// ***************************************************************************

	// define BALL_TEST in preprocessor definitions to run unit test      

	#include "ball.h"

   bool Ball::initialized = false; // initialize class static in global space
   char * scolor[3] = {"white", "green", "blue"};

   //------------------------------------------------------------------------
   // Ball Methods
   //------------------------------------------------------------------------

   Ball::Ball ( unsigned char n )  // Constructor
   //------------------------------------------------------------------------
   // initializes the object
   // seeds random generator for color & lotto
   //------------------------------------------------------------------------
   {                       
      int cnum = 0; // color number to effect a random color assignment
   
      if ( !initialized )  // safe programming
      {    
         srand( time( NULL ) );  // seed rand with time 
         initialized = true;     // for random ball color
      }   
   
      cnum = rand() % 3;         // 0, 1, 2
   
      picked = false;            // start false
   
      number = n;                // 0 - not a lotto number
      strncpy( color, scolor[ cnum ], COLOR_SIZE ); // copy random color
   
   #ifdef BALL_TEST // comment to output
      cout << "Constructor: created an instance of Ball" << endl;   
   #endif
   
   } // end Constructor


   Ball::~Ball() // Destructor 
   //------------------------------------------------------------------------
   // cleanup the object
   //------------------------------------------------------------------------
   {
      // no dynamic memory to deallocate
   #ifdef BALL_TEST // comment to output
      cout << "Destructor: Ball object has dropped out of scope" << endl; 
   #endif
   } // end Destructor 


   void   
   Ball::Show( void )
   //------------------------------------------------------------------------
   // shows the number of the ball and its picked state
   //------------------------------------------------------------------------
   {
      unsigned char shownum = 0;  // ball assignment
   
   #ifdef BALL_TEST
      char * pick_value[2] = {"FALSE", "TRUE"}; // for output
   
      shownum = Get_number(); // get the number value
   
      cout  << "Ball value = " << setw( 3 ) 
         << static_cast<int>(shownum) << ":  "
         << " color = " << setw( 6 ) << color << ":  ";
   
      if ( picked )
      {
         cout << "picked = "<< setw( 5 ) << pick_value[1] << endl;
      }
      else
      {
         cout << "picked = "<< pick_value[0] << endl;
      }
   
   #else
   
      shownum = Get_number(); // get the number value
   
      cout  << setw( 5 ) << static_cast<int>(shownum);
   
   #endif
   
   } // end Ball::Show( void )


   void   
   Ball::Set_number ( unsigned char n )
   //------------------------------------------------------------------------
   // sets the number of the ball
   //------------------------------------------------------------------------
   {
      if ( n <= 49 ) // error check
      {
         number = n;
      }
      else
      {
         cerr << "ERROR:  Number out of bounds" << endl;
         exit( 1 ); // error - exit program
      }
   
   } // end Ball::Set_number ( unsigned char n )


   unsigned char   
   Ball::Get_number ( void )
   //------------------------------------------------------------------------
   // returns the number of the ball
   //------------------------------------------------------------------------
   {  
      return( number ); // simple accessor method returns number
   
   } // end Ball::Get_number ( void )


   void   
   Ball::Reset ( void )
   //------------------------------------------------------------------------
   // Resets the picked state of the ball to false
   //------------------------------------------------------------------------
   {
      picked = false;  // simply resets picked state to false
   
   } // end Ball::Reset ( void )


   bool   
   Ball::Pick ( void )
   //------------------------------------------------------------------------
   // Returns true if ball is not yet picked and sets ball picked;
   // returns false otherwise
   //------------------------------------------------------------------------
   {
      bool pickable = false;  // assume fail: ball is not pickable
   
      if ( !picked )
      {                    // if not picked  
         picked = true;    // set picked 
         pickable = true;
      }
   
      return ( pickable ); 
   
   } // end Ball::Pick ( void )



#ifdef BALL_TEST

   void        
   main ( void )
   //------------------------------------------------------------------------
   // unit test of all the methods of ball
   //------------------------------------------------------------------------
   {                
      unsigned char num = ( rand() % 49 ) + 1;  // ball assingment
   
      cout << "Unit Test of class Ball\n\n";
   
      cout << "Create Ball on the stack by direct declaration:" << endl;
   
      Ball stackBall( num ); // creates instance of ball; fires constructor
      // stack based object; is bound to scope
      // object destructed when it drops from scope
   
      stackBall.Show();
      num = ( rand() % 49 ) + 1;
      cout << "Set the number to " << static_cast<int>( num ) << ":" 
         << endl;
      stackBall.Set_number( num );
      stackBall.Show();
   
      if ( stackBall.Pick() == true )
      {
         cout << "Pick the ball:" << endl;
         stackBall.Show();
      }
      else
      {
         cerr << "ERROR: failed to pick the ball" << endl;
      }   
   
      cout << "Reset the ball: " << endl;
      stackBall.Reset();
      stackBall.Show();      
   
      // object on the heap NOT bound by scope
      Ball * ptr2Ball = 0;    // Rule 1: initialize to NULL 
   
      cout << "\nCreate Ball on the heap by dynamic allocation:" << endl;
   
      ptr2Ball = new Ball();  // allocates memory and fires constructor
   
      if ( ptr2Ball == 0 )  // Rule 2: always test pointers
      {
         cerr << "ERROR: not able to allocate memory" << endl;
         exit ( 1 );
      }
   
      ptr2Ball->Show(); // show the new ball
      num = ( rand() % 49 ) + 1;
      cout << "Set the number to " << static_cast<int>(num) << ":" 
         << endl;
      ptr2Ball->Set_number( num );  // set new number and show
      ptr2Ball->Show();
   
      if ( ptr2Ball->Pick() == true )
      {
         cout << "Pick the ball" << endl; // pick the ball
         ptr2Ball->Show();
      }
      else
      {
         cerr << "ERROR: failed to pick the ball" << endl;
      }
   
      cout << "Reset the ball: " << endl;  
      ptr2Ball->Reset(); // reset the ball and show
      ptr2Ball->Show();
   
      // Heap based objects must be destroyed by the program
      cout << "Program destroys Ball on the heap:" << endl;
      if ( ptr2Ball )  // Rule 3: cleanup memory
      {
         delete( ptr2Ball );
         ptr2Ball = 0;
      }
   
      cout << "\nEnd Ball Unit Test: stack Ball object dropping out of scope" 
         << endl;
   
   } // end main

#endif

// end ball.cpp