/******************************************************************************\
		 Bettina McCormick, April 1998, CSIII, Dr. Leeson, Assignment #3

		 Tree Investigation Project  - Due Date: Wednesday April 29, 1998

		 Main Program:  trees.c  Version 2.1 - last updated  April 27, 1998

	 This program investigates two different tree structures:
	 the Binary Search Tree (BST) and the Two-Three Tree (TTT).

\******************************************************************************/
 #include "bstree.h"
 #include "tttree.h"
 #include "trees.h" 

/******************************************************************************/
/***************************   F U N C T I O N S  *****************************/

/*--------------------------------------------------------------   tttree.c   */
/*----------------------T T T _N E W N O D E _ I N S E R T--------------------*\

 Purpose: called when a new node is needed for inserting

 Pre-Condition : Follow Parameters;

 Post-Condition: the data has been inserted and the node initialized

 Return: a pionter to the node created
\*----------------------------------------------------------------------------*/
   tttnode *ttt_newnode_insert(int data)
   {
      tttnode *newnode;
   
      if ((newnode = (tttnode*) malloc(sizeof(tttnode))) == NULL)
      {
         printf("Not enough memory to allocate tttnode\n");
         exit(1);
      } /* end if */
      newnode -> leftval = data;
      newnode -> rightval = NADA;
      newnode -> parent = SINGLE;
      newnode -> return_value = NADA;
      newnode -> left = NULL;
      newnode -> center = NULL;
      newnode -> right = NULL;
      newnode -> extra = NULL;
      newnode -> new_sibling = NULL;
   
      return newnode;
   } /* end ttt_newnode_insert */

/*-----------------------------T T T _ I S L E A F----------------------------*\
 Purpose: checks if the node passed is a leaf

 Pre-Condition : Follow Parameters;

 Post-Condition: node has been checked for leaf property

 Return: TRUE if it is a leaf; FALSE otherwise
\*----------------------------------------------------------------------------*/
   int ttt_isleaf(tttnode *node)
   {
      int isleaf_flag = FALSE;
   
      if (node -> left == NULL) /* if this pointer is NULL, then no kids */
      {
         isleaf_flag = TRUE;
      }
      return isleaf_flag;
   } /* end ttt_isleaf */

/*---------------------T T T _ S I M P L E _ I N S E R T----------------------*\

 Purpose: inserts a data value in the rightval field of the node passed

 Pre-Condition : Follow Parameters;

 Post-Condition: the data has been inserted only - might not be in the
					  right relationship

 Return: EXIT_SUCCESS

\*----------------------------------------------------------------------------*/
   int ttt_simple_insert(tttnode *node, int data)
   {
      node -> rightval = data;     /* insert the value */
      node -> parent = MARRIED;  /* indicate node now has 2 values */
   
      return EXIT_SUCCESS;
   } /* end ttt_simple_insert */

/*---------------------T T T _ A D J U S T _ V A L U E S----------------------*\

 Purpose: Swaps the values in a node to there proper relatinship.

 Pre-Condition : Follow Parameters;

 Post-Condition: the data has been swaped if necessary

 Return: EXIT_SUCCESS

\*----------------------------------------------------------------------------*/
   int ttt_adjust_values(tttnode *node)
   {
      int temp;  /* used for swaping the values */
   
      if (node -> leftval > node -> rightval)
      {
         temp = node -> rightval;
         node -> rightval = node -> leftval;
         node -> leftval = temp;
      }
      return EXIT_SUCCESS;
   } /* end ttt_adjust_values */

/*-----------------------------T T T _ I N S E R T----------------------------*\

 Purpose: This routine calls the TTT routines: it inserts a new value into the
			 2-3 tree - keeping it balanced.

 Pre-Condition : Follow Parameters;

 Post-Condition: the data has been inserted in the TTTree.

 Return: a pointer to the root of the tree.

 Comments: Whew - this baby was tough! -  I did a combination of recursion
 and parent pointing; I can't wait to learn C++!  I think it will make things
 like this alot easier!!


\*----------------------------------------------------------------------------*/
   tttnode *ttt_insert(tttnode *root, int data)
   {
      tttnode *temp_sibling = NULL;   /* my temporary pointers - to avoid passing */
      tttnode *big_sibling = NULL;
      tttnode *newroot = NULL;
   
      if (root == NULL)     /* if tree is empty */
      {
         root = ttt_newnode_insert(data);
      }
      
      else if (ttt_isleaf(root) == TRUE)
      {
         if (root -> parent == SINGLE)
         {                /* if you are at a leaf and there is space to insert */
            ttt_simple_insert(root, data);
            ttt_adjust_values(root);
         }
         else if (root -> parent == MARRIED)
         {               /* if you are at a leaf and there is NO space to insert */
            if ((temp_sibling = (tttnode*) malloc(sizeof(tttnode))) == NULL)
            {
               printf("Not enough memory to allocate tttnode\n");
               exit(1);
            } /* end if */
            if ( data > root -> rightval)
            {                                           /* create a new node */
               temp_sibling -> leftval = data;
               temp_sibling -> return_value = root -> rightval;
               root -> rightval = NADA;
            }
            else if (data < root -> leftval)
            {
               temp_sibling -> leftval = root -> rightval;
               root -> rightval = NADA;
               root -> parent = SINGLE;
               temp_sibling -> return_value = root -> leftval;
               root -> leftval = data;
            }
            else
            {
               temp_sibling -> leftval = root -> rightval;
               temp_sibling ->return_value = data;
               root -> parent = SINGLE;
               root -> rightval = NADA;
            }
            temp_sibling -> rightval = NADA;
            temp_sibling -> left = NULL;
            temp_sibling -> center = NULL;
            temp_sibling -> right = NULL;
            temp_sibling -> extra = NULL;
            temp_sibling -> new_sibling = NULL;
            root -> parent = temp_sibling -> parent = SINGLE;
            root -> new_sibling = temp_sibling;   /* asign this node to a pointer
         					in the root to facilitate passing */
         }
      } /* end if is a leaf */
      else
      {     						/* if not a leaf */
         if (root -> parent == SINGLE)
         {   /* if you are NOT at a leaf and only two kids - keep going (recursion) */
            if (data < root -> leftval)
            {
               ttt_insert(root -> left, data);
            }
            else
            {
               ttt_insert(root -> center, data);
            }
         }
         else if (root -> parent == MARRIED)
         {    /* if you are NOT at a leaf and three kids - keep going (recursion) */
            if (data < root -> leftval)
            {
               ttt_insert(root -> left, data);
            }
            else if (data > root -> rightval)
            {
               ttt_insert(root -> right, data);
            }
            else
            {
               ttt_insert(root -> center, data);
            }
         }
      } /* end else not a leaf */
   
      if (root -> new_sibling != NULL)      /* if a node node was created */
      {                                 /*- leaf split, receive promoted value  */
         if (root -> extra == NULL)
         {
            if ((newroot = (tttnode*) malloc(sizeof(tttnode))) == NULL)
            {
               printf("Not enough memory to allocate tttnode\n");
               exit(1);
            } /* end if */
            newroot -> leftval = root -> new_sibling -> return_value;
            root -> new_sibling -> return_value = NADA;
            newroot -> rightval = NADA;
            newroot -> return_value = NADA;
            newroot -> left = root;
            newroot -> left -> extra = newroot;
            newroot -> center = root -> new_sibling;
            root -> new_sibling = NULL;
            newroot -> center -> extra = newroot;
            newroot -> right = NULL;
            newroot -> extra = NULL;
            newroot -> new_sibling = NULL;
            newroot -> parent = SINGLE;
            root = newroot;
            root -> center -> extra = root;
            root -> left -> extra = root;
         }
         else if ((root -> extra != NULL) && (root -> extra -> parent == SINGLE))
         {           						/* make third child */
            root -> extra -> new_sibling = root -> new_sibling;
            root -> new_sibling = NULL;
            root = root -> extra;
            if (root -> new_sibling -> return_value < root -> leftval)
            {                                      /* build left */
               root -> right  =  root -> center;
               root -> rightval = root -> leftval;
               root -> leftval  = root -> new_sibling -> return_value;
               root -> center = root -> new_sibling;
               root -> left -> extra = root;
               root -> center -> extra = root;
               root -> right -> extra = root;
               root -> new_sibling = NULL;
               root -> parent = MARRIED;
            }
            else
            {                                    /* build right */
               root -> rightval = root -> new_sibling -> return_value;
               root -> new_sibling -> return_value = NADA;
               root -> right = root -> new_sibling;
               root -> right -> extra = root;
               root -> new_sibling = NULL;
               root -> parent = MARRIED;
            }
         }
         else if ((root -> extra != NULL) && (root -> extra -> parent == MARRIED))
         {			/* split node, promote middle value */
            if ((big_sibling = (tttnode*) malloc(sizeof(tttnode))) == NULL)
            {
               printf("Not enough memory to allocate tttnode\n");
               exit(1);
            } /* end if  */
            big_sibling -> rightval = NADA;
            big_sibling -> return_value = NADA;
            big_sibling -> left = NULL;
            big_sibling -> center = NULL;
            big_sibling -> right = NULL;
            big_sibling -> extra = NULL;
            big_sibling -> new_sibling = NULL;
            root -> parent = big_sibling -> parent = SINGLE;
            root -> extra -> new_sibling = root -> new_sibling;
            root -> new_sibling = NULL;
            root = root -> extra;
            if (root -> new_sibling -> return_value < root -> leftval)
            {  /* left child split promote */
               big_sibling -> leftval = root -> rightval;
               root -> rightval = NADA;
               big_sibling -> return_value = root -> leftval;
               root -> leftval = root -> new_sibling -> return_value;
               root -> new_sibling -> return_value = NADA;
               root -> parent = SINGLE;
               big_sibling -> left = root -> center;
               big_sibling -> left -> extra = big_sibling;
               big_sibling -> center = root -> right;
               big_sibling -> center -> extra = big_sibling;
               root -> center = root -> new_sibling;
               root -> center -> extra = root;
               root -> left -> extra = root;
               root -> new_sibling = NULL;
               root -> right = NULL;
               root -> new_sibling = big_sibling;
            }
            else if (root -> new_sibling -> return_value > root -> rightval)
            { /* right child split, promote  */
               big_sibling -> leftval = root -> new_sibling -> return_value;
               root -> new_sibling -> return_value = NADA;
               big_sibling -> return_value = root -> rightval;
               root -> parent = SINGLE;
               big_sibling -> left = root -> right;
               root -> right = NULL;
               big_sibling -> left -> extra = big_sibling;
               big_sibling -> center = root -> new_sibling;
               root -> new_sibling = NULL;
               big_sibling -> center -> extra = big_sibling;
               root -> new_sibling = big_sibling;
            }
            else
            {
               big_sibling -> leftval = root -> rightval;
               root -> rightval = NADA;
               root -> parent = SINGLE;
               big_sibling -> return_value = root -> new_sibling -> return_value;
               root -> new_sibling -> return_value = NADA;
               big_sibling -> left = root -> new_sibling;
               root -> new_sibling = NULL;
               big_sibling -> left -> extra = big_sibling;
               big_sibling -> center = root -> right;
               root -> right = NULL;
               big_sibling -> center -> extra = big_sibling;
               root -> new_sibling = big_sibling;
            }
         }
      }
      return root;
   } /* end ttt_insert */


/*-----------------------------T T T _ S E A R C H----------------------------*\

 Purpose: searches for the data passes in the tree ans counts compares

 Pre-Condition : Follow Parameters: TTT_cmprs is initialized to zero
					  prior to passing into funtion

 Post-Condition: compares are counted

 Return: TRUE if found, FALSE otherwise

 Comments - this follows the algorithm provided by Dr. Leeson

\*----------------------------------------------------------------------------*/
   int ttt_search(tttnode *root, int data, int *ttt_cmprs)
   {
      int found_flag = FALSE;
   
      *ttt_cmprs = *ttt_cmprs + 1;
      if (root == NULL)
      {
         return found_flag;            /* value not found */
      }
      if (data == root -> leftval)
      {
         found_flag = TRUE;
         return found_flag;     				/* value found */
      }
      if ((root -> parent == MARRIED) && (data == root -> rightval))
      {
         found_flag = TRUE;
         return found_flag;
      }
      if (data < root -> leftval)
      {
         return ttt_search(root -> left, data, ttt_cmprs);
      }
      else if (root -> parent == SINGLE)
      {
         return ttt_search(root -> center, data, ttt_cmprs);
      }
      else if (data < root -> rightval)
      {
         return ttt_search(root -> center, data, ttt_cmprs);
      }
      else
      {
         return ttt_search(root -> right, data, ttt_cmprs);
      };
   
   }

/*---------------------------T T T _ T R A V E R S E--------------------------*\

 Purpose: does a modified traversal of TTT and prints results to test file;
			 this is the traversal Dr. Leeson explained in class

 Pre-Condition : Follow Parameters: file is open for writing

 Post-Condition: data is written to file and file is still open

 Return: void

\*----------------------------------------------------------------------------*/
   void ttt_traverse(tttnode *root, FILE **fp)
   {
      if (root != NULL)
      {
         if (root -> parent == SINGLE)
         {
            fprintf(*fp, "%d\n", root -> leftval);
         }
         else
         {
            fprintf(*fp, "%d %d\n", root -> leftval, root -> rightval);
         }
         if (root -> left != NULL)
         {
            fprintf(*fp, "going left\n");
            ttt_traverse(root -> left, fp);
            fprintf(*fp, "returning from left\n");
         }
         if (root -> center != NULL)
         {
            fprintf(*fp, "going center\n");
            ttt_traverse(root -> center, fp);
            fprintf(*fp, "returning from center\n");
         }
         if (root -> right != NULL)
         {
            fprintf(*fp, "going right\n");
            ttt_traverse(root -> right, fp);
            fprintf(*fp, "returning from right\n");
         }
      } /* end if */
   } /* end bst_inorder */




/*-------------------------------------------------------------  end tttree.c */

