/* title  a2a.c
/*****************************************************************************\

/*------------------------------- A U T H O R  -------------------------------*\
	Bettina McCormick
	CAP 4453 - Robot Vision - Fall 1998
	Assignment 2A  - Due Date: October 3, 1998
	Version 1.5  - last updated  October 1, 1998
\*----------------------------------------------------------------------------*/

/*------------------------ D E S C R I P T I O N  ----------------------------*\
  This program detects the edges of an image.  It uses the Canny method of
  smoothing to create the difference operator used for convolution with the
  image.  This is the magnitude of the gradient and this data is the first
  outputted image; it should resemble the first image outputted in the Sobel
  process (a1a.c). Then candidates for edges are detected by looking for peaks
  in the direction of the gradient.	 This data is the second image that is
  outputted.  Then using this canditate array and the first image data and two
  threshold values a final image is created with only the candidates that are
  above the high threshold or connected to a candidate above the high threshold.
  The output for this program is 3 files: the magnitude image, the candidate
  image and the final image.  This is in the raw data form and then Dr. Lobo's
  "rawtpgm" program converts it to a .pgm file. Then xv is used to see the
  converted images.  See the "go" file.
\*----------------------------------------------------------------------------*/

/*--------------------- R E G A R D I N G _ I N P U T  -----------------------*\
  Input for this program was an image provided by the instructor.  The
  dimensions of this image are 256 by 256.  The user enters the input on the
  command line followed by the three output files, the decimal value for SIGMA,
  the decimal value for the HIGH threshold and the decimal value for the LOW
  threshold. (7 comand line arguments in that order)
\*----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h> /* not recognized by UNIX - use the -lm commmand instead */


#define  X_DIMENSION	  		256    	/* x dim of image */
#define  Y_DIMENSION	  		256    	/* y dim of image */
#define  SCALE_FACTOR  		255		/* maximum value for pixel */
#define  FLEX_WIDTH	    	 80    	/* size of M-H_difopr mask */
#define  EDGE_CHECK	     	  1	 	/* radius used in checking edge connection */
#define  TAN225	   	0.4142		/* tangent of 22.5 degrees */
#define  TAN675	   	2.4142		/* tangent of 67.5 degrees */
#define  ZERO             	  0	 	/* for easier readablity */
#define  ALMOSTzero   .0000001		/* to replace zero in the denominator */
#define  EDGE	  	     		255	 	/* for easier readablity */
#define  TRUE					  1  		/* for easier readablity */
#define  FALSE					  0  		/* for easier readablity */
#define  ON					 255  		/* for easier readablity */
#define  OFF					  0  		/* for easier readablity */

   int main (int argc, char ** argv)
   {
   /*----------------  V A R I A B L E  D E C L A R A T I O N S  ----------------*/
   
      FILE 	*infp1,                   /* pointer to input file */
         *outfp1,                  /* pointer to output file */
         *outfp2,                  /* pointer to output file */
         *outfp3,                  /* pointer to output file */
         *fopen();                 /* pointer to function */
   
      int 	Ay, Bx, Iy, Jx, 	/* for incrementation */
      MORE,					/* boolean for while loop */
      MASK_RADIUS,  		/* calculated with input SIGMA */
      HALF_WIDTH;		 	/* calculated with constant FLEX_WIDTH */
   
   
      int   input_image[Y_DIMENSION][X_DIMENSION];
   				/* 2D array to hold input image */
   
      int   scaled_image[Y_DIMENSION][X_DIMENSION];
   				/* 2D array to hold scaled image */
   
      int   candidate_image[Y_DIMENSION][X_DIMENSION];
   			/* 2D array to hold candidate image */
   
      int   canny_image[Y_DIMENSION][X_DIMENSION];
   			/* 2D array to hold final canny image*/
   
      double 	x_p1, y_p1, x_p2, y_p2 = 0,   /* for parts of the gaussian equation */
      SIGMA,         	/* read in on command line */
      XYsum_SQRD = 0, 	/* x squared + y squared */
      SIGMA_SQRD = 0,  	/* sigma squared */
      POWER = 0,       	/* the power that e gets raised to */
      SLOPE = 0,			/* slope of the line (y/x) */
      x_temp = 0,			/* temp variable for x part of convolution */
      y_temp = 0,     	/* temp variable for y part of convolution */
      x_canny_difopr[FLEX_WIDTH][FLEX_WIDTH],  /* X direction difopr mask */
      y_canny_difopr[FLEX_WIDTH][FLEX_WIDTH],  /* Y direction difopr mask */
      x_part[Y_DIMENSION][X_DIMENSION], /* convolution with x mask */
      y_part[Y_DIMENSION][X_DIMENSION], /* convolution with y mask */
      xysq_temp = 0,	/* x squared + Y squared */
      mag_max = 0,	/* maximum value in them  magnitude array */
      scale_temp = 0,       /* temp variable for scaling process */
      HIGH = 0,    	 /* read in on command line -high threshold */
      LOW  = 0,    	 /* read in on command line -low threshold */
      magnitude[Y_DIMENSION][X_DIMENSION];   /* image after convolution */
   
   /*--------------------------  B E G I N  M A I N  ----------------------------*/
      if(argc < 8)   /* test arguments */
      {
         fprintf(stderr, "\nYou must enter 7 command-line arguments following ");
         fprintf(stderr, "the executable:\n\n");
         fprintf(stderr,
                "C:\\>executable  input  output  output  output  sigma  high  low\n");
         exit(1);
      };
   
   /*-----------------------------------------------------------------open files */
      if((infp1 = fopen(argv[1], "r")) == NULL)   /* opens 1st input file */
      {
         fprintf(stderr, "Error opening input file.");
         exit(1);
      };
   
      if((outfp1 = fopen(argv[2], "w")) == NULL)   /* opens output file */
      {
         fprintf(stderr, "Error opening first output file.");
         exit(1);
      };
   
      if((outfp2 = fopen(argv[3], "w")) == NULL)   /* opens output file */
      {
         fprintf(stderr, "Error opening second output file.");
         exit(1);
      };
   
      if((outfp3 = fopen(argv[4], "w")) == NULL)   /* opens output file */
      {
         fprintf(stderr, "Error opening third output file.");
         exit(1);
      };
   
   /*---------------------------------assign command line arguments to variables */
      SIGMA = atof(argv[5]);        /* SIGMA read from command line */
      if (SIGMA == ZERO)
      {
         printf("You entered 0.0 for sigma - sigma must be greater than zero.");
         printf("Program halted - run again with a valid sigma.");
         exit(1);
      }
   
      HIGH = atof(argv[6]);        /* HIGH read from command line */
      LOW = atof(argv[7]);        /* LOW read from command line */
   
   /*----------------------------------------------fill canny x & y difopr masks */
      SIGMA_SQRD = (SIGMA * SIGMA);
      MASK_RADIUS = (3 * SIGMA);
      HALF_WIDTH = (FLEX_WIDTH / 2);
   	/* this is the x and y parts of the equation for the Laplacian of a
   	Gaussian broken into parts p1, p2 for x and y respectively */
      for (Iy = (-MASK_RADIUS); Iy <= MASK_RADIUS; Iy++)
      {
         for (Jx = (-MASK_RADIUS); Jx <= MASK_RADIUS; Jx++)
         {
            XYsum_SQRD = ((Jx * Jx) + (Iy * Iy));
            POWER =  (XYsum_SQRD/(2 * SIGMA_SQRD));
            x_p1 = (-Jx/SIGMA_SQRD);
            x_p2 =  exp(-POWER);
            y_p1 = (-Iy/SIGMA_SQRD);
            y_p2 =  exp(-POWER);
            x_canny_difopr[(HALF_WIDTH + Iy)][(HALF_WIDTH + Jx)] = (x_p1 * x_p2);
            y_canny_difopr[(HALF_WIDTH + Iy)][(HALF_WIDTH + Jx)] = (y_p1 * y_p2);
         } /* end for Jx */
      } /* end for Iy */
   /*--------------------------------------------------------fill image array(s) */
      for (Ay = 0; Ay < Y_DIMENSION; Ay++)
      {
         for (Bx = 0; Bx < X_DIMENSION; Bx++)
         {
            input_image[Ay][Bx]  =  getc (infp1);
            input_image[Ay][Bx]  &= 0377;
         } /* end for Bx */
      } /* end for Ay */
   
   /***********************     BEGIN PROCESSING IMAGE    ************************/
   
   /*--------------------------------------------------------compute convolution */
      for (Ay = MASK_RADIUS; Ay < (Y_DIMENSION - MASK_RADIUS); Ay++)
      {
         for (Bx = MASK_RADIUS; Bx < (X_DIMENSION- MASK_RADIUS); Bx++)
         {
            for (Iy = - MASK_RADIUS; Iy <= MASK_RADIUS; Iy++)
            {
               for (Jx = - MASK_RADIUS; Jx <= MASK_RADIUS; Jx++)
               {
                  x_temp = x_temp + (input_image[Ay + Iy][Bx + Jx] *
                                    x_canny_difopr[(HALF_WIDTH + Iy)][(HALF_WIDTH + Jx)]);
                  y_temp = y_temp + (input_image[Ay + Iy][Bx + Jx] *
                                    y_canny_difopr[(HALF_WIDTH + Iy)][(HALF_WIDTH + Jx)]);
               } /* end for Jx */
            } /* end for Iy */
            x_part[Ay][Bx] = x_temp;
            y_part[Ay][Bx] = y_temp;
            x_temp = 0;
            y_temp = 0;
         } /* end for Bx */
      } /* end for Ay */
   
   /*-------------------------------------------------------take the derivative  */
   /*---------------------------square root of (x_part squared + y_part squared) */
      for (Ay = 0; Ay < Y_DIMENSION; Ay++)
      {
         for (Bx = 0; Bx < X_DIMENSION; Bx++)
         {
            xysq_temp = ((x_part[Ay][Bx] * x_part[Ay][Bx])  +
                        (y_part[Ay][Bx] * y_part[Ay][Bx]));
            magnitude[Ay][Bx] = sqrt(xysq_temp);
         } /* end for Bx */
      } /* end for Ay */
   
   
   /*-------------------------------------------scan magnitude for maximum value */
      for (Ay = 0; Ay < Y_DIMENSION; Ay++)
      {
         for (Bx = 0; Bx < X_DIMENSION; Bx++)
         {
            if (magnitude[Ay][Bx] > mag_max)
               mag_max =  magnitude[Ay][Bx];
         } /* end for Bx */
      } /* end for Ay */
   
   /*--------------------------------------------------scale image to (0 to 255) */
   
      for (Ay = 0; Ay < Y_DIMENSION; Ay++)
      {
         for (Bx = 0; Bx < X_DIMENSION; Bx++)
         {
            scale_temp = ((magnitude[Ay][Bx] / mag_max) * SCALE_FACTOR);
            scaled_image[Ay][Bx] = (int)scale_temp;
            scale_temp = 0;
         } /* end for Bx */
      } /* end for Ay */
   
   
   /*--------------------------------------------------------output scaled image */
      for (Ay = 0; Ay < Y_DIMENSION; Ay++)
      {
         for (Bx = 0; Bx < X_DIMENSION; Bx++)
         {
            fprintf(outfp1,"%c",(char)scaled_image[Ay][Bx]);
         } /* end for Bx */
      } /* end for Ay */
   
   
   /*----------------------------------------------------------detect candidates */
      for (Ay = 0; Ay < Y_DIMENSION; Ay++)
      {
         for (Bx = 0; Bx < X_DIMENSION; Bx++)
         {
            if (x_part[Ay][Bx] == ZERO)
               x_part[Ay][Bx] = ALMOSTzero;
            SLOPE = (y_part[Ay][Bx] / x_part[Ay][Bx]);
            if ((SLOPE > (-TAN225)) && (SLOPE < TAN225))
            {
               if ((magnitude[Ay][Bx] > magnitude[Ay][Bx + 1])
                  && (magnitude[Ay][Bx] > magnitude[Ay][Bx - 1]))
                  candidate_image[Ay][Bx] = ON;
            }
            else if ((SLOPE > (-TAN675)) && (SLOPE < (-TAN225)))
            {
               if ((magnitude[Ay][Bx] > magnitude[Ay + 1][Bx - 1])
                  && (magnitude[Ay][Bx] > magnitude[Ay - 1][Bx + 1]))
                  candidate_image[Ay][Bx] = ON;
            }
            else if ((SLOPE < TAN675) && (SLOPE > TAN225))
            {
               if ((magnitude[Ay][Bx] > magnitude[Ay + 1][Bx + 1])
                  && (magnitude[Ay][Bx] > magnitude[Ay - 1][Bx - 1]))
                  candidate_image[Ay][Bx] = ON;
            }
            else
            {
               if ((magnitude[Ay][Bx] > magnitude[Ay - 1][Bx])
                  && (magnitude[Ay][Bx] > magnitude[Ay + 1][Bx]))
                  candidate_image[Ay][Bx] = ON;
            }
         } /* end for Bx */
      } /* end for Ay */
   
   
   /*------------------------------------------output candidate image */
      for (Ay = 0; Ay < Y_DIMENSION; Ay++)
      {
         for (Bx = 0; Bx < X_DIMENSION; Bx++)
         {
            fprintf(outfp2,"%c",(char)candidate_image[Ay][Bx]);
         } /* end for Bx */
      } /* end for Ay */
   
   
   /*--------------------------------------use double threshold to connect edges */
   /*---------------------------------------------------------------------pass 1 */
      for (Ay = 0; Ay < Y_DIMENSION; Ay++)
      {
         for (Bx = 0; Bx < X_DIMENSION; Bx++)
         {
            if (candidate_image[Ay][Bx] == ON)
            {
               if (magnitude[Ay][Bx] > HIGH)
               {
                  canny_image[Ay][Bx] = EDGE;
                  candidate_image[Ay][Bx] = OFF;
               }
               else if (magnitude[Ay][Bx] < LOW)
               {
                  candidate_image[Ay][Bx] = OFF;
               }
            }
         } /* end for Bx */
      } /* end for Ay */
   /*---------------------------------------------------------------------pass 2 */
      MORE = TRUE;
      while (MORE == TRUE)
      {
         MORE = FALSE;
         for (Ay = 0; Ay < Y_DIMENSION; Ay++)
         {
            for (Bx = 0; Bx < X_DIMENSION; Bx++)
            {
               if (candidate_image[Ay][Bx] == ON)
               {
                  for (Iy = -EDGE_CHECK; Iy <= EDGE_CHECK; Iy++)
                  {
                     for (Jx = -EDGE_CHECK; Jx <= EDGE_CHECK; Jx++)
                     {
                        if (canny_image[Ay + Iy][Bx + Jx] == EDGE)
                        {
                           MORE = TRUE;
                           canny_image[Ay][Bx] = EDGE;
                           candidate_image[Ay][Bx] = OFF;
                        }
                     
                     } /* end for Jx */
                  } /* end for Iy */
               
               }
            } /* end for Bx */
         } /* end for Ay */
      } /* end while more = true */
   
   /*------------------------------------------output final canny image */
      for (Ay = 0; Ay < Y_DIMENSION; Ay++)
      {
         for (Bx = 0; Bx < X_DIMENSION; Bx++)
         {
            fprintf(outfp3,"%c",(char)canny_image[Ay][Bx]);
         } /* end for Bx */
      } /* end for Ay */
   /*************************     END PROCESSING IMAGE    ************************/
   
   
   /*--------------------------------------------------------------  close files */
      fclose(infp1);
      fclose(outfp1);
      fclose(outfp2);
      fclose(outfp3);
   
      return EXIT_SUCCESS;
   } /* end main a2a.c */

