/* ************************************************************************ *\
      Bettina McCormick, March 1998, CSIII, Dr. Leeson, Assignment #2

      Hash Table Comparison Project  - Due Date: March 11, 1998

      Closed Hash File: chash.c Version 5.1- last updated March 5, 1998

      This file contains the functions for the closed hashing.
\* ************************************************************************ */

#include "hashes.h"
#include "chash.h"

   closed_hashtable Init_ClosedHashTable(int ts, int bs)
   /*  Takes the table size and bin size, mallocs space and initializes the
    tabsize to table size, the binsize to the bin size. Calculates overflow
    size and initializes all slots to VIRGIN. Returns the table.  */
   {
      closed_hashtable H;
   
      if ((H.hashtab = (int *) malloc((ts + EXTRA) * sizeof(int))) == NULL)
      {
         printf("Not enough memory to allocate prime array buffer\n");
         exit(1);
      } /* end if */
   
      memset(H.hashtab, VIRGIN, sizeof(int) * (ts + EXTRA));
   
      H.binsize = bs;
      H.oflowsize = (ts + EXTRA) - (((ts / bs) * bs) + bs);
      H.tabsize = ts;
      H.lastoflow = (((H.tabsize / H.binsize) * H.binsize) + H.binsize);
      H.oflowstart = (((H.tabsize / H.binsize) * H.binsize) + H.binsize);
   
      return H;
   } /* end  Init_ClosedHashTable */


   int Hash_ClosedHashTable(closed_hashtable *H, int r)
   /*  Takes table and the value to be hashed; returns the hash index.  */
   {
      int index;
   
      index = r % H -> tabsize;
   
      return index;
   } /* end Hash_ClosedHashTable */


   int Find_ClosedHashTable(closed_hashtable *H, int r)
   /*  Takes table and the value to be found; returns TRUE if found.  */
   /* This function is not really necessary to this program b/c data file
   will contain no duplicates - however, I wanted to wirte it for the
   logic, and in keeping with a more normal hash process. */
   {
      int found = FALSE,
      tracker = 0,
      i,
      x;
   
      x = Hash_ClosedHashTable(H, r);
   
      if (H -> hashtab[x] == VIRGIN)
         found = FALSE;
      else
      {
         while(((x+1) % H->binsize) != 0)
         {
            if (H->hashtab[x] == VIRGIN)
               found = FALSE;
            else if (H->hashtab[x] == r)
               found = TRUE;
            x++;
            tracker++;
         } /* end while */
         if (found == FALSE)
         {
            tracker++;
            if (H->hashtab[x] == VIRGIN)
               found = FALSE;
            else if (H->hashtab[x] == r)
               found = TRUE;
         }
         if (found == FALSE)
         {
            for (i = (x -(H->binsize - 1)); i <= (x - tracker); i++)
            {
               if (H->hashtab[x] == r)
                  found = TRUE;
            }
         } /* end if */
         if (found == FALSE)
         {
            for (i = H->oflowstart; i <= H->lastoflow; i++)
            {
               if (H->hashtab[i] == r)
                  found = TRUE;
            }
         }
      } /* end else */
      if (found == TRUE)
         return TRUE;
      else
         return FALSE;
   } /* end  Find_ClosedHashTable */


   int Insert_ClosedHashTable(closed_hashtable *H, int r)
   /*  Takes the hash table and the value to be inserted;
    calls Hash_OpenHashTable and Find_OpenHashTable;
    post condition - pointer to new node has been inserted in hash table.  */
   {
   
      int i,
      x,
      insertflag = FALSE,
      tracker = 0;
   
      x = Hash_ClosedHashTable(H, r);
   
      if (Find_ClosedHashTable(H, r))
         return EXIT_SUCCESS;
      else
      {
         while((((x+1) % H->binsize) != 0) && (insertflag == FALSE))
         {
            if (H->hashtab[x] == VIRGIN)
            {
               H->hashtab[x] = r;
               insertflag = TRUE;
               x++;
            } /* end if VIRGIN */
            else
            {
               x++;
               tracker++;
            } /* end else */
         } /* end while */
         if ((insertflag == FALSE) && (H->hashtab[x] == VIRGIN))
         {
            H->hashtab[x] = r;
            insertflag = TRUE;
         }
         else
            tracker++;
         if (insertflag == FALSE)
         {
            for (i = (x -(H->binsize - 1)); i <= (x - tracker); i++)
            {
               if ((H->hashtab[i] == VIRGIN) && insertflag == FALSE)
               {
                  H->hashtab[i] = r;
                  insertflag = TRUE;
               }
            }
         } /* end if */
         if (insertflag == FALSE)
         {
            H->hashtab[H->lastoflow] = r;
            H->lastoflow++;
         }
      } /* end else */
      return EXIT_SUCCESS;
   } /* end Insert_ClosedHashTable */


   int OutputStats_Closed(closed_hashtable *H)
   /* Takes the hash table and output the statistics:  # of filled slots
   of the table, percentage filed, longest list length, and average
   list length. */
   {
      static int header = FALSE;
   
      int oflowcount = 0,
      average_entries = 0,
      binflag = FALSE,
      binsfilled = 0,
      bincount,
      x = 0,
      i,
      *avgdata;
   
      if ((avgdata=(int *)calloc(H->binsize + 1 * sizeof(int),
         H->binsize + 1 * sizeof(int))) == NULL)
      {
         printf("Not enough memory to allocate prime array buffer\n");
         exit(1);
      } /* end if */
   
      while (x < H->oflowstart)
      {
         bincount = 0;
         binflag = FALSE;
         for (i = x; i < (x + H->binsize); i++)
         {
            if (H->hashtab[i] != VIRGIN)
            {
               bincount++;
               binflag = TRUE;
            }
         } /* end for */
         if (binflag == TRUE)
         {
            binsfilled++;
            avgdata[bincount]++;
         } /* end if */
         x = x + H->binsize;
      } /* end while */
   
      for (i = 1; i <= H->binsize; i++)
      {
         if (avgdata[i] > 0)
         {
            avgdata[i] = avgdata[i] * i;
            average_entries += (float)avgdata[i];
         }
      } /* end for j */
   
      average_entries = (float)average_entries / binsfilled;
   
   
      if (H->lastoflow > H->oflowstart)
         oflowcount = (H->lastoflow - H->oflowstart);
      else
         oflowcount = 0;
   
   
      if (header == FALSE)
      {
         printf("\n\n\t\t*******\tClosed Hash Statistics\t*******\t\t\n\n");
         printf("Table_Size  Bin_Size      Bins_Filled    Average/Bin");
         printf("    Overflow_Size\n");
         header = TRUE;
      } /* end if */
   
      printf("\n%-12d%-14d%-15d%-15d%-14d ",
            H->tabsize, H->binsize, binsfilled, average_entries, oflowcount);
      if ( H->binsize == 7)
         printf("\n"); 
   
   
      return EXIT_SUCCESS;
   
   } /* end OutputStats_Closed */


   int ClosedHash(char *input, int ts)
   /* Takes pointer to the file pointer and table size; reads in a
   file of non- duplicating integers, creates hash table and prints
   statistics. */
   {
   
      int filedata = 0,     /* holds the table index of the value hashed */
      bs;                  /* bin size */
   
      FILE *fp;
   
      closed_hashtable closedtable;
   
   /*************************************************************************/
   
      if( (fp = fopen(input, "r")) == NULL)/* opens argv[1] file */
      {
         fprintf(stderr, "Error opening file.");
         exit(1);
      };
   
      rewind(fp);
      bs = 3;
      closedtable = Init_ClosedHashTable(ts, bs);
      while (feof(fp)== FALSE)
      {                    /* reads single char from file, saves as int */
         fscanf(fp, "%ld\n", &filedata);
         Insert_ClosedHashTable(&closedtable, filedata);
      }; /* end while */
      OutputStats_Closed(&closedtable);
   
      rewind(fp);
      bs = 5;
      closedtable = Init_ClosedHashTable(ts, bs);
   
      while (feof(fp)== FALSE)
      {                       /* reads single char from file, saves as int */
         fscanf(fp, "%ld\n", &filedata);
         Insert_ClosedHashTable(&closedtable, filedata);
      }; /* end while */
      OutputStats_Closed(&closedtable);
   
   
      rewind(fp);
      bs = 7;
      closedtable = Init_ClosedHashTable(ts, bs);
   
      while (feof(fp)== FALSE)
      {                       /* reads single char from file, saves as int */
         fscanf(fp, "%ld\n", &filedata);
         Insert_ClosedHashTable(&closedtable, filedata);
      }; /* end while */
      OutputStats_Closed(&closedtable);
   
   
      fclose(fp);
   
      free(closedtable.hashtab);
   
      return EXIT_SUCCESS;
   } /* end ClosedHash */


/*******************************   end chash.c   ****************************/
