/* ************************************************************************** *\
       Bettina McCormick, March 1998, CSIII, Dr. Leeson, Assignment #2

          Hash Table Comparison Project  - Due Date: March 11, 1998

       Open Hash File: ohash.c Version 5.1- last updated March 5, 1998

      This file contains the functions for the open hashing.
\* ************************************************************************** */

#include "hashes.h"
#include "ohash.h"

   open_hashtable Init_OpenHashTable(int ts)
   /*  Takes the tablesize, mallocs space and initializes the tabsize to
    table size, the filled count to zero, and the longest linked list
    to zero.  Returns the table.  */
   {
      open_hashtable H;
   
      int i;
   
      if ((H.hashtab = (hashnode **) malloc(ts * sizeof(hashnode))) == NULL)
      {
         printf("Not enough memory to allocate prime array buffer\n");
         exit(1);
      } /* end if */
   
      for (i = 0; i < ts; i++)
      {
         H.hashtab[i] = NULL;
      }
   
      H.tabsize = ts;
      H.filled = 0;
      H.longest = 0;
   
      return H;
   } /* end  Init_OpenHashTable */


   int Hash_OpenHashTable(open_hashtable *H, int r)
   /*  Takes table and the value to be hashed; returns the hash index.  */
   {
      int index;
   
      index = r % H -> tabsize;
   
      return index;
   } /* end Insert_OpenHashTable */

   int Find_OpenHashTable(open_hashtable *H, int r)
   /*  Takes table and the value to be found; returns TRUE if found.  */
   /* This function is not really necessary to this program b/c data file
   will contain no duplicates - however, I wanted to wirte it for the
   logic, and in keeping with a more normal hash process. */
   {
      int found = 0,
      x;
   
      hashnode *tempptr;
   
      x = Hash_OpenHashTable(H, r);
      tempptr = H -> hashtab[x];
   
      if (H -> hashtab[x] == NULL)
         found = FALSE;
      else if (tempptr != NULL)
      {
         while ((tempptr != NULL) && (found != TRUE))
         {
            if (tempptr -> intvalue == r)
               found = TRUE;
            
            else
               tempptr = tempptr -> next;
         } /* end while */
      } /* end else */
   
      if (found == TRUE)
         return TRUE;
      else
         return FALSE;
   } /* end Find_OpenHashTable */


   int Insert_OpenHashTable(open_hashtable *H, int r)
   /*  Takes the hash table and the value to be inserted;
    calls Hash_OpenHashTable and Find_OpenHashTable;
    post condition - pointer to new node has been inserted in hash table.  */
   {
      int c = 0,
      x;
   
      hashnode  *tempptr,
         *newnode;
   
      x = Hash_OpenHashTable(H, r);
   
      tempptr = H -> hashtab[x];
   
      if (Find_OpenHashTable(H, r))
         return EXIT_SUCCESS;
      
      else if (H -> hashtab[x] == NULL)
      {
         newnode = (hashnode*) malloc(sizeof(hashnode));
         newnode -> intvalue = r;
         newnode -> next = NULL;
         H -> hashtab[x] = newnode;
         c++;
         H -> filled++;
         H -> hashtab[x] -> numnode = c;
         if (H -> longest < c)
            H -> longest = c;
      } /* end else if NULL */
      else
      {
         while (tempptr != NULL)
         {
            c++;
            tempptr = tempptr -> next;
         } /* end while */
         c++;
         newnode = (hashnode*) malloc(sizeof(hashnode));
         newnode -> intvalue = r;
         newnode -> numnode = c;
         newnode -> next = H -> hashtab[x];
         H -> hashtab[x] = newnode;
         if (H -> longest < c)
            H -> longest = c;
      } /* end else */
   
      return EXIT_SUCCESS;
   } /* end Insert_OpenHashTable */

   void Free_OpenHashTable(open_hashtable *H)
   /*  Takes table and and frees the space allocated for it and all the
    pointers in it */
   {
      int x;
   
      hashnode *tempptr;
   
      for(x = 0; x < H -> tabsize; x++)
      {
         if (H -> hashtab[x] != NULL)
         {
            tempptr = H -> hashtab[x];
         
            while (H -> hashtab[x] != NULL)
            {
               H -> hashtab[x] = tempptr -> next;
               free(tempptr);
               tempptr = H -> hashtab[x];
            } /* end while */
         } /*end if */
      } /*end for */
      for(x = 0; x < H -> tabsize; x++)
      {
         free(H -> hashtab[x]);
      } /*end for */
      free(H -> hashtab);
      return;
   } /* end Free_OpenHashTable */



   int OutputStats_Open(open_hashtable *H)
   /* Takes the hash table and output the statistics:  # of filled slots
   of the table, percentage filed, longest list length, and average
   list length. */
   {
      static int header = FALSE;
   
      int *avgdata;
   
      int i, j;
   
      float  percent_full = 0, /* percentage of the table that is full */
      average_list = 0; /* averge length of all the linked lists */
   
      percent_full = ((float)H->filled / (float)H->tabsize) * 100;
   
      if ((avgdata=(int *)calloc(H->longest * sizeof(int),
         H->longest * sizeof(int))) == NULL)
      {
         printf("Not enough memory to allocate prime array buffer\n");
         exit(1);
      } /* end if */
   
      for (j = 1; j <= H -> longest; j++)
      {
         for (i = 0; i < H -> tabsize; i++)
         {
            if (H -> hashtab[i] != NULL)
            {
               if (H -> hashtab[i] -> numnode == j)
               {
                  avgdata[j]++;
               }
            } /* end if */
         } /* end for i */
         avgdata[j] = avgdata[j] * j;
      } /* end for j */
   
      for (j = 1; j < (H -> longest + 1); j++)
      {
         average_list += (float)avgdata[j];
      } /* end for j */
   
      average_list = (float)average_list / H->filled;
   
      if (header == FALSE)
      {
         printf("\n\n\t\t*******\tOpen Hash Statistics\t*******\t\t\n\n");
         printf("Table_Size  Percent_Full  Slots_Filled   Longest_List");
         printf("   Average_List\n");
         header = TRUE;
      } /* end if */
   
      printf("\n%-12d%-14.3f%-15d%-14d ",
            H->tabsize, percent_full, H->filled, H->longest);
      printf("%-15.3f", average_list);
   
      return EXIT_SUCCESS;
   
   } /* end OutputStats */


   int OpenHash(char *input, int ts)
   /* Takes pointer to the file pointer and table size; reads in a
   file of non- duplicating integers, creates hash table and prints
   statistics. */
   {
      int filedata = 0;     /* holds the table index of the value hashed */
   
      FILE *fp;
   
      open_hashtable opentable;
   
   /***************************************************************************/
   
      if( (fp = fopen(input/* argv[1] */, "r")) == NULL)/* opens argv[1] file */
      {
         fprintf(stderr, "Error opening file.");
         exit(1);
      };
   
   
      opentable = Init_OpenHashTable(ts);
   
      while (feof(fp)== FALSE)
      {                          /* reads single char from file, saves as int */
         fscanf(fp, "%u\n", &filedata);
      #ifdef DEBUG
      printf("\n%d", filedata);
      #endif
         Insert_OpenHashTable(&opentable, filedata);
      }; /* end while */
      fclose(fp);
   
      OutputStats_Open(&opentable);
   
      Free_OpenHashTable(&opentable);
   
      return EXIT_SUCCESS;
   } /* end OpenHash */

/*****************************   end ohash.c   ********************************/

