/* ************************************************************************** */
/*      			Bettina McCormick, January 1998, CSIII, Dr. Leeson	      */
/*                                                                            */
/*       Huffman Code Compression Project  - Due Date: February 9, 1998       */
/*                                                                            */
/*           decode.c - Version 9.1 - last updated January 30, 1998           */
/*                                                                            */
/*                                                                            */
/*          This program takes the two files generated by encode.c:           */
/*      creates a frequecy table, a huffman tree using a heap for the         */
/*		  implementation, decodes the .cod file to a intermediate file of     */
/*		  chars, and then decode the  intermediate file; it outputs a final   */
/*		  text file which should match the original file given to encode.c.   */
/*                                                                            */
/* ************************************************************************** */

#include "heap.h"

   int main(int argc, char **argv)
   {
      int  filedata,			        	       /* to store data from file */
		   filenum = 0,/* to store size of original ASCII characters in chars */
		   j, k;                                     /* for incrementation */
   
      unsigned int freqtable[MAXARRAY] = {0};       /* to hold frequencies */
   
      unsigned long int
      n = sizeof(char) * BYTE1,                        /* for bit unmasking */
      charnum,                                       /* to count chars read */
      l = 0,
      codelength = 0;         /* to store number of char in coded .mid file */
   
      unsigned char chdata,                            /* for bit unmasking */
      mask;
   
      char codeary[MAXARRAY] = {'\0'},/* dummy array to simulate pass by value */
      nullstring[MAXCODE] = {'\0'};
   
      FILE  *in_fp, *in2_fp, *out_fp, *tmp_fp, *dbug_fp;  /* pointers to files */
   
   
      node node1, *ptrnode, *ptrnode1, *ptrnode2;
											/* to hold file data and treelinks*/
   
      heap heap1, *pheap;        /* to hold the nodes in a heap implementaion */
   
      code huffcode[MAXARRAY];        /* array of code structures (strings) */
   
   
   /************************       begin main       ****************************/
   
      if( (in_fp = fopen(argv[1], "rb")) == NULL) /* opens argv[1].tre file */
      {
         fprintf(stderr, "Error opening file.");
         exit(1);
      };
   
      if(argc < 4)   /* test arguments */
      {
         fprintf(stderr, "\nYou must enter 3 command-line arguments following ");
         fprintf(stderr, "the executable:\n\n");
         fprintf(stderr,
                "C:\\>C:\\>executable  .tre_file   .cod_file   output_file \n");
         exit(1);
      }
   
   /*******************  make freqtable from argv[1].tre file  ****************/
   
      while (filenum < MAXARRAY)
      {                        /* reads single char from file, saves as int */
         fscanf(in_fp,"%d", &filedata);
         if (filedata != '0')
            freqtable[filenum] = filedata;     /* puts freqs in freqtable */
         filenum++;
      }; /* end while */
   
   #ifdef DEBUG
   for (j = 0; j < MAXARRAY; j++)
   {
   if (freqtable[j] != 0)
     printf("%c  %d\n", j, freqtable[j]);
   }/* end for */
   #endif
   
      fscanf(in_fp,"%d", &filenum);
   
      fclose(in_fp);				               /* closes argv[1].tre file */
   
   /***************************        make heap      *************************/
   
      HeapInit(&heap1);
   
      for (k=0; k<(MAXARRAY); k++)
      {                      /* creates a node and puts pointer to it in heap */
         if (freqtable[k] !=0)
         {
            ptrnode = (node*) malloc(sizeof(node));
            ptrnode -> fvalue = freqtable[k];
            ptrnode -> cdata = (char)(k);
            ptrnode -> scode[0] = '\0';
            ptrnode -> leftc = NULL;
            ptrnode -> rightc = NULL;
            HeapInsert (&heap1, ptrnode, heap1.hcount+1);
            heap1.hcount++;
         }  /* end if */
      }   /* end for */
   
   #ifdef DEBUG
   HeapDisplay(&heap1, heap1.hcount);
   #endif
   
   /***************************    make Huffman tree   ************************/
   
      while (heap1.hcount > 1) /* removes nodes from heap and builds hufftree */
      {
         ptrnode1 = DeleteRoot(&heap1);
         heap1.hcount--;
         ptrnode2 = DeleteRoot(&heap1);
         heap1.hcount--;
         ptrnode = (node*) malloc(sizeof(node));
         ptrnode -> fvalue = (ptrnode1 -> fvalue + ptrnode2 -> fvalue);
         ptrnode -> cdata = '*';
         ptrnode -> scode[0] = '\0';
         ptrnode -> leftc = ptrnode1;
         ptrnode -> leftc -> scode[0] = '0';
         ptrnode -> leftc -> scode[1] = '\0';
         ptrnode -> rightc = ptrnode2;
         ptrnode -> rightc -> scode[0] = '1';
         ptrnode -> rightc -> scode[1] = '\0';
         HeapInsert (&heap1, ptrnode, heap1.hcount+1);
         heap1.hcount++;
      } /*end while */
   
   #ifdef DEBUG
   printf ("%s","\nthis be the tree:\n");
   PreorderTraversal(ptrnode);
   #endif
   
   /******************    output Huffman code to an array   *******************/
   
      strcpy(huffcode[MAXARRAY].codestring, nullstring);/*initialize HuffCode array*/
      RetrieveCode(ptrnode, codeary, huffcode);/* output code to HuffCode array */
   
   /** Huffcode array used with fretable to generate encoded .mid file length **/
      for (j = 0; j < MAXARRAY; j++)
      {
         if (strlen(huffcode[j].codestring) > 0)
         {
            codelength += (strlen(huffcode[j].codestring)* freqtable[j]);
         } /* end for */
      } /* end for */
   
   #ifdef DEBUG
   printf("\n%d\n", codelength);
   #endif
   
   /*** convert binary bits to char display of code and output to .mid file ***/
   
      if( (in2_fp = fopen(argv[2], "rb")) == NULL) /* opens argv[2].cod file */
      {
         fprintf(stderr, "Error opening file.");
         exit(1);
      };
      tmp_fp = tmpfile();
   
   
      mask = 1;
      mask <<= 7;                /* sets codebits bit value to '10000000' */
      l = 0;
      charnum = 0;
   
      while (l < codelength)
      {
         chdata = getc(in_fp);
         charnum++;
         l += BYTE1;
         for (j = 0; j < n; ++j)
         {
            fputc( ((chdata & mask) == 0) ? '0' : '1', tmp_fp );
            chdata <<= 1;
         }
      
      } /* end while */
   
      fclose(in2_fp);   			               /* closes arg[2].cod file */
   
   
   /******** decode temp file and print a decoded file to .txt file  **********/
      rewind(tmp_fp);
      if ( (out_fp = fopen(argv[3], "w")) == NULL)/* opens argv[3]output file */
      {
         fprintf(stderr, "Error opening file.");
         exit(1);
      };
   
      l = 0;
      while (l < codelength)
      {
         DecodeNode(&l, ptrnode, tmp_fp, out_fp);
      } /* end while */
   
      fclose(out_fp);    			                  /* closes file */
   
      printf("\n\n\tYour file has been succesfully decoded to \"%s\"\n", argv[3]);
      printf("\n***********\tThis is soooooo cooooool!!!\t***********\n\n");
   
      return EXIT_SUCCESS;
   }
/*************************   end decode program  ******************************/


