/* title mh.c
/*****************************************************************************\

/*------------------------------- A U T H O R  -------------------------------*\
	Bettina McCormick
	CAP 4453 - Robot Vision - Fall 1998
	Assignment 1B - MarrHildrethEdge - Due Date: September 12, 1998
	Version 1.8  - last updated  September 10, 1998
\*----------------------------------------------------------------------------*/

/*------------------------ D E S C R I P T I O N  ----------------------------*\
  This program detects the edges of an image.  It uses convolution and the
  Marr-Hildreth method of smoothing (Laplacian of a Gaussian) to create the
  difference operator used for the image.  This convolution is the first
  outputted image.  Then this convolution is scaled and scanned for changes
  from positive to negative - thus detecting an edge.  This is the second
  image that is outputted.  The output for this program is 2 files:
  the convolution image and the positive to negative image.  This is in the
  raw data form and then  Dr. Lobo's rawtpgm program converts it to a .pgm
  file. Then xv is used to see the finished image.  See the "go" file.
\*----------------------------------------------------------------------------*/

/*--------------------- R E G A R D I N G _ I N P U T  -----------------------*\
  Input for this program was an image provided by the instructor.  The
  dimensions of this image are 256 by 256.  The user enters the input on the
  command line followed by the two output files and the decimal value for SIGMA
  and the decimal value for the tolerance. (5 comand line arguments in that 
  order)
\*----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h> /* not recognized by UNIX - use the -lm commmand instead */


#define  X_DIMENSION	  256    /* x dim of image */
#define  Y_DIMENSION	  256    /* y dim of image */
#define  SCALE_FACTOR   255		/* maximum value for pixel */
#define  FLEX_WIDTH	   80    /* size of M-H_difopr mask */
#define  SIGN_CHECK	    1	 /* radius used in checking sign change */
#define  ZERO               0	 /* for easier readablity */
#define  POSITIVE	    1	 /* for easier readablity */
#define  NEGATIVE          -1	 /* for easier readablity */
#define  EDGE	  	  255	 /* for easier readablity */


   int main (int argc, char ** argv)
   {
   /*----------------  V A R I A B L E  D E C L A R A T I O N S  ----------------*/
   
      FILE 	*infp1,                   /* pointer to input file */
         *outfp1,                  /* pointer to output file */
         *outfp2,                  /* pointer to output file */
         *fopen();                 /* pointer to function */
   
      int 	Ay, Bx, Iy, Jx, /* for incrementation */
      MASK_RADIUS,	/* calculated with input SIGMA */
      HALF_WIDTH;	/* calculated with constant FLEX_WIDTH */
   
   
      int   input_image[Y_DIMENSION][X_DIMENSION];
   				/* 2D array to hold input image */
   
      int   scaled_image[Y_DIMENSION][X_DIMENSION];
   				/* 2D array to hold scaled image */
   
      int   signed_image[Y_DIMENSION][X_DIMENSION];
   			/* 2D array to hold signed image */
   
      int   POStoNEG_image[Y_DIMENSION][X_DIMENSION];
   			/* 2D array to hold positve to negative image*/
   
      double 	p1, p2, p3 = 0,   /* for parts of the gaussian equation */
      SIGMA,         	/* read in on command line */
      XYsum_SQRD = 0, 	/* x squared + y squared */
      SIGMA_SQRD = 0,  	/* sigma squared */
      POWER = 0,       	/* the power that e gets raised to */
      M_H_temp = 0,     /* temp variable for equation */
      M_H_difopr[FLEX_WIDTH][FLEX_WIDTH],  /* M-H_difopr mask */
      conv_max = 0,         /* max value of convolution image */
      conv_min = 0,         /* min value of convolution image */
      scale_max = 0,        /* maximum of the abs value of the min or max */
      scale_temp = 0,       /* temp variable for scaling process */
      TOLERANCE = 0,    	 /* read in on command line */
      convolution[Y_DIMENSION][X_DIMENSION]; 
   				/* image after convolution */
   
   /*--------------------------  B E G I N  M A I N  ----------------------------*/
      if(argc < 5)   /* test arguments */
      {
         fprintf(stderr, "\nYou must enter 5 command-line arguments following ");
         fprintf(stderr, "the executable:\n\n");
         fprintf(stderr,
                "C:\\>executable  input  output  output  sigma  tolerance\n");
         exit(1);
      };
   
   /*-----------------------------------------------------------------open files */
      if((infp1 = fopen(argv[1], "r")) == NULL)   /* opens 1st input file */
      {
         fprintf(stderr, "Error opening input file.");
         exit(1);
      };
   
      if((outfp1 = fopen(argv[2], "w")) == NULL)   /* opens output file */
      {
         fprintf(stderr, "Error opening first output file.");
         exit(1);
      };
   
      if((outfp2 = fopen(argv[3], "w")) == NULL)   /* opens output file */
      {
         fprintf(stderr, "Error opening second output file.");
         exit(1);
      };
   
   /*-------------------------------------------------------fill M_H_difopr mask */
      SIGMA = atof(argv[4]);        /* SIGMA read from command line */
      SIGMA_SQRD = (SIGMA * SIGMA);
      MASK_RADIUS = (3 * SIGMA);
      HALF_WIDTH = (FLEX_WIDTH / 2);
   	/* this is the equation for the Laplacian of a Gaussian
   		broken into parts p1, p2 and p3*/
      for (Iy = (-MASK_RADIUS); Iy <= MASK_RADIUS; Iy++)
      {
         for (Jx = (-MASK_RADIUS); Jx <= MASK_RADIUS; Jx++)
         {
            XYsum_SQRD = ((Jx * Jx) + (Iy * Iy));
            POWER =  (XYsum_SQRD/(2 * SIGMA_SQRD));
            p1 = (-1/SIGMA_SQRD);
            p2 = (2 - (XYsum_SQRD/SIGMA_SQRD));
            p3 = exp(-POWER);
            M_H_difopr[(HALF_WIDTH + Iy)][(HALF_WIDTH + Jx)] = (/*p1 * */p2 * p3);
         /* p1 is a constant and not necessary
         	in the calculation for our purposes */
         } /* end for Jx */
      } /* end for Iy */
   /*--------------------------------------------------------fill image array(s) */
      for (Ay = 0; Ay < Y_DIMENSION; Ay++)
      {
         for (Bx = 0; Bx < X_DIMENSION; Bx++)
         {
            input_image[Ay][Bx]  =  getc (infp1);
            input_image[Ay][Bx]  &= 0377;
         } /* end for Bx */
      } /* end for Ay */
   
   /***********************     BEGIN PROCESSING IMAGE    ************************/
   
   /*--------------------------------------------------------compute convolution */
      for (Ay = MASK_RADIUS; Ay < (Y_DIMENSION - MASK_RADIUS); Ay++)
      {
         for (Bx = MASK_RADIUS; Bx < (X_DIMENSION- MASK_RADIUS); Bx++)
         {
            for (Iy = -MASK_RADIUS; Iy <= MASK_RADIUS; Iy++)
            {
               for (Jx = -MASK_RADIUS; Jx <= MASK_RADIUS; Jx++)
               {
                  M_H_temp = M_H_temp + (input_image[Ay + Iy][Bx + Jx] *
                                        M_H_difopr[(HALF_WIDTH + Iy)][(HALF_WIDTH + Jx)]);
               } /* end for Jx */
            } /* end for Iy */
            convolution[Ay][Bx] = M_H_temp;
            M_H_temp = 0;
         } /* end for Bx */
      } /* end for Ay */
   
   /*------------------------------scan convolution for maximum & minimum values */
      for (Ay = 0; Ay < Y_DIMENSION; Ay++)
      {
         for (Bx = 0; Bx < X_DIMENSION; Bx++)
         {
            if (convolution[Ay][Bx] > conv_max)
               conv_max = convolution[Ay][Bx];
            if (convolution[Ay][Bx] < conv_min)
               conv_min = convolution[Ay][Bx];
            scale_max = conv_max;
            if (fabs(conv_min) > conv_max)
               scale_max = fabs(conv_min);
         	/* set scale_max to the greater of the abs value of the two */
         } /* end for Bx */
      } /* end for Ay */
   
   /*--------------------------------------------------scale image to (0 to 255) */
   /* adding the abs max value to the convolution array  shifts all
   	the values to positive */
      for (Ay = 0; Ay < Y_DIMENSION; Ay++)
      {
         for (Bx = 0; Bx < X_DIMENSION; Bx++)
         {
            scale_temp =(((convolution[Ay][Bx] + scale_max) * SCALE_FACTOR)/
                        (2.0 * scale_max));
            scaled_image[Ay][Bx] = (int)scale_temp;
            scale_temp = 0;
         } /* end for Bx */
      } /* end for Ay */
   /*------------------------------------scan image for tolerance and mark signs */
      TOLERANCE = atof(argv[5]);        /* TOLERANCE read from command line */
      for (Ay = 0; Ay < Y_DIMENSION; Ay++)
      {
         for (Bx = 0; Bx < X_DIMENSION; Bx++)
         {
            if (fabs(convolution[Ay][Bx]) < TOLERANCE)
               signed_image[Ay][Bx] = ZERO;
            else if(convolution[Ay][Bx] > TOLERANCE)
               signed_image[Ay][Bx] = POSITIVE;
            else
               signed_image[Ay][Bx] = NEGATIVE;
         } /* end for Bx */
      } /* end for Ay */
   
   /*----------------------scan signed_image for shift from positive to negative */
      for (Ay = SIGN_CHECK; Ay < (Y_DIMENSION - SIGN_CHECK); Ay++)
      {
         for (Bx = SIGN_CHECK; Bx < (X_DIMENSION - SIGN_CHECK); Bx++)
         {
            if (signed_image[Ay][Bx] == ZERO)
            {
               if ((signed_image[Ay-1][Bx] * signed_image[Ay+1][Bx]) < ZERO)
                  POStoNEG_image[Ay][Bx] = EDGE;   /* Y axis */
               else if ((signed_image[Ay][Bx-1] * signed_image[Ay][Bx+1]) < ZERO)
                  POStoNEG_image[Ay][Bx] = EDGE;   /* X axis */
               else if ((signed_image[Ay-1][Bx-1]*signed_image[Ay+1][Bx+1]) < ZERO)
                  POStoNEG_image[Ay][Bx] = EDGE;   /* Negative slope */
               else if ((signed_image[Ay+1][Bx-1]*signed_image[Ay-1][Bx+1]) < ZERO)
                  POStoNEG_image[Ay][Bx] = EDGE;   /* Positive slope */
            } /*  end  if (signed_image[Ay][Bx] = ZERO) */
            else if (signed_image[Ay][Bx] == POSITIVE)
            {
               for (Iy = -SIGN_CHECK; Iy <= SIGN_CHECK; Iy++)
               {
                  for (Jx = -SIGN_CHECK; Jx <= SIGN_CHECK; Jx++)
                  {
                     if (signed_image[Ay + Iy][Bx + Jx] == NEGATIVE)
                        POStoNEG_image[Ay][Bx] = EDGE;
                  } /* end for Jx */
               } /* end for Iy */
            } /*  end  if (signed_image[Ay][Bx] = POSITIVE) */
         } /* end for Bx */
      } /* end for Ay */
   /*************************     END PROCESSING IMAGE    ************************/
   
   /*--------------------------------------------------------output scaled image */
      for (Ay = 0; Ay < Y_DIMENSION; Ay++)
      {
         for (Bx = 0; Bx < X_DIMENSION; Bx++)
         {
            fprintf(outfp1,"%c",(char)scaled_image[Ay][Bx]);
         } /* end for Bx */
      } /* end for Ay */
   
   
   /*------------------------------------------output positive to negative image */
      for (Ay = 0; Ay < Y_DIMENSION; Ay++)
      {
         for (Bx = 0; Bx < X_DIMENSION; Bx++)
         {
            fprintf(outfp2,"%c",(char)POStoNEG_image[Ay][Bx]);
         } /* end for Bx */
      } /* end for Ay */
   
   /*--------------------------------------------------------------  close files */
      fclose(infp1);
      fclose(outfp1);
   
      return EXIT_SUCCESS;
   } /* end main */

