/*
        Simulator Project for COP4600
        Original Design:  Dr. David Workman, 1990

        Revised (1993)
                Tim Hughey
                Mark Stephens

      	OSDEFS.H 
*/

/* SCRIPT PROGRAM CODES */
/* refer to PROGM_FILE[] and pgmidtab[] */
#define   EDITOR      0
#define   PRINTER     1
#define   COMPILER    2
#define   LINKER      3
#define   USER        4
#define   BOOT        5
#define   LOGOFF      6

/* EVENT CODES */
#define   LOGON       0
#define   STARTIO     1
#define   SIO         1
#define   WAITIO      2
#define   WIO         2
#define   ENDIO       3
#define   EIO         3
#define   PGMEND      4
#define   TIMEEVENT   5
#define   SEGFAULT    6
#define   ADRFAULT    7

/* INSTRUCTION OPERATION CODES */
#define  OP_SIO       0
#define  OP_CPU       1
#define  OP_WIO       1
#define  OP_REQ       2
#define  OP_JUMP      3
#define  OP_SKIP      4
#define  OP_END       5    /* note: DEVICE CODES start at 6 */

/* SCHEDULER CODES */
#define   FCFS        0
#define   SJN         1
#define   HPRN        2
#define   RNDRBN      3
#define   MIN         60            /* seconds per minute   */
#define   SEC         1000000000    /* nanosec per second   */
#define   SECOVR2     500000000     /* SEC/2                */
#define   SECOVR20    50000000      /* SEC/20               */
#define   MSEC        1000000       /* nanosec per millisec */
#define   mSEC        1000          /* nanosec per microsec */
#define   NSEC        1
#define   DSEC        1.0e9

/* SIMTIME is the type used to represent Simulation time. */
struct simtime    {
       unsigned long  seconds;
       unsigned long  nanosec;
                  };


/* Logical program addresses in JUMP and REQ instructions are of ADDR_TYPE */
struct addr_type  {
       int           segment;
       unsigned int  offset;
                  };

union opernd_type {
       struct addr_type  address;      /* REQ and JUMP instructions*/
       unsigned int      count;        /* SKIP         instruction */
       unsigned long     burst;        /* SIO, WIO,END instructions*/
       unsigned long     bytes;        /* device       instructions*/
                  };

/* MEMORY is an array of INSTR_TYPE */
struct instr_type {
       unsigned   char     opcode;
       union opernd_type   operand;
                  };

struct segment_type{
       unsigned char accbits;  /* bits defining seg. access */
       unsigned int  seglen;   /* segment length            */
       unsigned long membase;  /* memory address of segment */
                  };

extern struct rb_type;             /* declared below */
extern struct rb_list;             /* declared below */
extern struct pcb_list;            /* declared below */


struct state_type {
       unsigned char      mode;        /* Cpu mode == 0x01 (priviledged)  */
       struct addr_type   pc;          /* program counter(segment,offset) */
                  };

struct pcb_type   {
       char               status;
                                    /* 'N' = NEW, 'R' = READY, 'A' = ACTIVE,
                                       'B' = BLOCKED, 'D' = DONE,
                                       'T' = TERMINATED--LOGGED OFF
                                    */
       char                flags;
                                     /*  bit      0 = set timer flag  */
                                     /*  bit      1 = new instr flag  */
                                     /*  bit      2 = new burst flag  */
                                     /*  bits   3-7 = not used        */
       char                user[5];  /* User terminal "Uxxx" */
       unsigned int        termnl;   /* index in termtable   */
       int                *script;   /* Pointer to process script */
       int                  pgmid;   /* index into script */
       struct segment_type *segtable; /* segment table pointer */
       unsigned int       segtab_len;/* segment table length  */
       struct state_type  cpu_save;  /* save area for cpu state when
                                          an interrupt occurs */
       struct simtime     logont;    /* time of logon     */
       struct simtime     blockedt;  /* last time BLOCKED */
       struct simtime     readyt;    /* last time READY   */
       struct simtime     runt;      /* last time ACTIVE  */
       unsigned long      sjnburst;  /* length of last burst */
       double             sjnave;    /* ave of last n bursts */
       unsigned long      instrleft; /* instructions remaining
                                        - for RR scheduling */
       unsigned long      sliceleft; /* time slice remaining
                                        - for RR scheduling */
       struct simtime     tblocked;  /* total time BLOCKED  */
       struct simtime     tready;    /* total time READY    */
       struct simtime     trun;      /* total time ACTIVE   */
       struct simtime     tlogon;    /* total time LOGGED ON */
       int		  served;    /* # of times job allocated to CPU */
       double             efficncy;  /* (tblocked+tactive)/tlogon*/
       struct rb_type     *wait;     /* IORB ptr when waiting
                                        for I/O completion */
       struct rb_list     *firstrb,  /* list of IORBs attached */
                          *lastrb;   /* to this process        */

       struct simtime	  waitq;     /* all device wait times for this user */
       struct simtime     busy;	     /* all device service times */
                  };

/* IORB (I/O Request Block ) */
struct rb_type    {
       char             status;
                               /* 'D' = done, 'A' = active, 'P' = pending */
       struct pcb_type  *pcb;  /* ptr. to requesting process             */
       int               dev;  /* devtable index of requested device     */
       struct simtime  queuet; /* time rb enters the device queue        */
       unsigned long    bytes; /* bytes to transfer                      */
       struct addr_type reqid; /* logical address of program instruction
                                  that generated request                 */
                  };



/* The system devtable is an array of the following type */
struct device_type{
       char               devid[5];    /* device name */
       struct   rb_type  *currb;       /* pointer to active IORB */
                                       /* status is indicated by currb */
       unsigned long int  npb;         /* device rate: nanosec/byte */
       unsigned long int  byps;        /* device rate: bytes/second */
       struct simtime     busy;        /* total busy time      */
       struct simtime     qwait;       /* total q-wait time    */
       struct simtime     response;    /* average response     */
       struct simtime     idle;        /* total idle time      */
       int                served;      /* no. served           */
       int                maxq;        /* max queue length     */
       int                qlen;        /* current queue length */
       double             utilize;     /* utilization          */
       struct rb_list     *head,*tail; /* device wait queue    */
       struct simtime     start;       /* time that I/O starts on a device */
                  };

struct cpu_type   {
       unsigned long       rate;          /* cpu rate in nanosec/instr.*/
       unsigned long       CPUBURST;      /* cpu rate in instr./sec    */
       unsigned char       mode;          /* == 0x01 (priviledged)     */
                                          /* == 0x00 (user)            */
       struct addr_type    pc;             /* program counter      */
       struct pcb_type    *actvpcb;        /* active process       */
       struct simtime      busy;           /* total busy time      */
       struct simtime      qwait;          /* total q-wait time    */
       struct simtime      response;       /* average response     */
       struct simtime      idle;           /* total idle time      */
       int                 served;         /* no. served           */
       int                 maxq;           /* max queue length     */
       int                 qlen;           /* current queue length */
       double              utilize;        /* utilization          */
       struct pcb_list    *ready;          /* ready list (head)    */
       struct pcb_list    *tail;           /* ready list (tail)    */
                  };


/* The TIMER device for RR scheduling */
struct timer_type {
       struct simtime TIME_OUT; /* time of next interrupt     */
       unsigned long  QUANTUM;  /* RR Quantum in instructions */
       struct simtime RRSLICE;  /* RR Quantum in (SEC,NSEC)   */
                  };

/* The event list is a doubly-linked list of elements of EVENT_TYPE */
struct event_type {
       struct simtime    time;
       int               event;
       int               agent;
       struct event_type *prev,*next;
                  };

/* for IORB list elements */
struct rb_list    {
       struct rb_type  *rb;
       struct rb_list  *next;
                  };

/* for lists like the CPU ready queue */
struct pcb_list   {
       struct pcb_type *pcb;
       struct pcb_list *next,*prev;
                  };

/* for maintaining list of free memory segments */
struct seg_list   {
        unsigned  long     segsize; /* could be all of MEM */
        unsigned  long     segptr;  /* index in MEM        */
        struct seg_list   *next;
                  };


