/* title  shapes.c
/*****************************************************************************\

/*------------------------------- A U T H O R  -------------------------------*\
	Bettina McCormick
	CAP 4453 - Robot Vision - Fall 1998
	Assignment 4 - Shapes - Due Date: October 5, 1998
	Version 2.8  - last updated  November 2, 1998
\*----------------------------------------------------------------------------*/

/*------------------------ D E S C R I P T I O N  ----------------------------*\
	This program uses the parametric equations for a super elipsoid to create
	shapes in 3 dimensions.
\*----------------------------------------------------------------------------*/

/*--------------------- R E G A R D I N G _ I N P U T  -----------------------*\
	There is no input except the command line arguements.  This outputs 5 files
   in raw format.
\*----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h> /* not recognized by UNIX - use the -lm commmand instead */

#define  X_DIMENSION	  		256    	/* x dim of image */
#define  Y_DIMENSION	  		256    	/* y dim of image */
#define  Z_DIMENSION	  		256    	/* Z dim of image */
#define  X_AXIS	  			1    		/* x axis of image */
#define  Y_AXIS	  			2    		/* y axis of image */
#define  Z_AXIS	  			3    		/* Z axis of image */
#define  RANGE			  		360    	/* max range for u and v  */
#define  U_STEP			  	0.0698	/* division step for 45 divisions of pi */
#define  V_STEP			  	0.1396 	/* division step for 45 divisions of 2pi */
#define  BRIGHT		      255      /* for easier readability */
#define  Pi			      	3.1415927      /* value for pi */
#define  ZERO			      0       	/* for easier readability */


   double sign (double SinCos_value);
/* takes the value for a double and returns 1 if positive or -1 if negative */

   void display (int outpic[Y_DIMENSION][X_DIMENSION], FILE **fp);
/* takes an array of int and a file pointer and out puts it to the file */

   void clear_xyz(double *x, double *y, double *z);

   void rotate (int axis, double ang, double *Rx, double *Ry, double *Rz);

   int main (int argc, char ** argv)
   {
   /*----------------  V A R I A B L E  D E C L A R A T I O N S  ----------------*/
   
      FILE 	*outfp1,                  /* pointer to first output file */
         *outfp2,                  /* pointer to second output file */
         *outfp3,                  /* pointer to third output file */
         *outfp4,                  /* pointer to fouth output file */
         *outfp5,                  /* pointer to fifth output file */
         *fopen();                 /* pointer to function */
   
      struct xyz {
         double 	x_coord,
         y_coord,
         z_coord;
      };
   
      typedef  struct xyz xyz_values;
   
      int 	i, j, k,    /* for incrementation */
      display_offset,	/* to shift image for display */
      index,
      imageX,
      imageY;
   
      int   first_image[Y_DIMENSION][X_DIMENSION];
   				/* 2D array to hold first image */
   
      int   sec_image[Y_DIMENSION][X_DIMENSION];
   				/* 2D array to hold 2nd image */
   
      int   thr_image[Y_DIMENSION][X_DIMENSION];
   				/* 2D array to hold 2nd image */
   
      int   forth_image[Y_DIMENSION][X_DIMENSION];
   				/* 2D array to hold 2nd image */
   
      int   fifth_image[Y_DIMENSION][X_DIMENSION];
   				/* 2D array to hold 2nd image */
   
      xyz_values XYZ[RANGE];
   					/* 2D array to hold x & y data */
   
      double  X_OFFSET,	  		  /* x offset to see 3Dness if image */
      Y_OFFSET,	  		  /* y offset to see 3Dness if image */
      Z_FORWARD,  		   /* translation component */
      MAGNIFY,          /* magnify factor */
      u,
      v,
      newX,             /* x after x_offset added */
      newY,             /* y after y_offset added */
      newZ,             /* z after z_forward added */
      t1x, t2y,				/* tapering parameters */
      b1, b2, b3,			/* bending parameters */
      P, Q, PHI, TAU,   /* twisting parameters */
      angle,				/* angle of rotation in rads */
      W = 0,            /* scale factor real value 0 <= w <= 1 */
      a1x = 0,          /* x-axis parameter */
      a2y = 0,          /* y-axis parameter */
      a3z = 0,          /* z-axis parameter */
      E1 = 0,           /* squareness parameter 1 >= 0 */
      E2 = 0,           /* squareness parameter 2 >= 0 */				
      Cu = 0,           /* cosine of u */
      Cv = 0,           /* cosine of v */
      Su = 0,           /* sine of u */
      Sv = 0,           /* sine of v */
      CuAbs = 0,           /* absolute value of cosine of u */
      CvAbs = 0,           /* absolute value of cosine of v */
      SuAbs = 0,           /* absolute value of sine of u */
      SvAbs = 0,           /* absolute value of sine of v */
      Cu2E1 = 0,           /* cosine of u to power of E1 */
      Cv2E2 = 0,           /* cosine of v to power of E2 */
      Su2E1 = 0,           /* sine of u to power of E1 */
      Sv2E2 = 0;           /* sine of v to power of E2 */
   
   
   
   /*--------------------------  B E G I N  M A I N  ----------------------------*/
   
      if(argc < 7)   /* test arguments */
      {
         fprintf(stderr, "\nYou must enter 6 command-line arguments following ");
         fprintf(stderr, "the executable:\n\n");
         fprintf(stderr,
                "C:\\>executable w  a1  a2  a3  E1  E2 \n");
         exit(1);
      };
   
   /*---------------------------------assign command line arguments to variables */
      W = atof(argv[1]);        /* read from command line */
      a1x = atof(argv[2]);      /* read from command line */
      a2y = atof(argv[3]);      /* read from command line */
      a3z = atof(argv[4]);      /* read from command line */
      E1 = atof(argv[5]);       /* read from command line */
      E2 = atof(argv[6]);       /* read from command line */
   /*----------------------------------------error check command line arguements */
      if (W < ZERO)
      {
         printf("You entered a value less than zero for W.\n");
         printf("Program halted - run again with a valaue between 0 and 1.");
         exit(1);
      }
      else if (W > 0.5)
      {
         printf("You entered a value greater than 0.5 for W.\n");
         printf("Program halted - run again with a value between 0 and 0.5.");
         exit(1);
      }
      if (E1 < ZERO)
      {
         printf("You entered a value less than zero for E1.\n");
         printf("Program halted - run again with value greater than zero.");
         exit(1);
      }
   
      if (E2 < ZERO)
      {
         printf("You entered a value less than zero for E2.\n");
         printf("Program halted - run again with value greater than zero.");
         exit(1);
      }
   /*-----------------------------------------------------------------open files */
      if((outfp1 = fopen("basic1.out", "w")) == NULL)   /* opens 1st output file */
      {
         fprintf(stderr, "Error opening first output file.");
         exit(1);
      };
   
   
      if((outfp2 = fopen("rotate2.out", "w")) == NULL)   /* opens 2nd output file */
      {
         fprintf(stderr, "Error opening first output file.");
         exit(1);
      };
   
      if((outfp3 = fopen("taper3.out", "w")) == NULL)   /* opens 3rd output file */
      {
         fprintf(stderr, "Error opening third output file.");
         exit(1);
      };
   
   
      if((outfp4 = fopen("bend4.out", "w")) == NULL)   /* opens 4th output file */
      {
         fprintf(stderr, "Error opening fourth output file.");
         exit(1);
      };
   
      if((outfp5 = fopen("twist5.out", "w")) == NULL)   /* opens 5th output file */
      {
         fprintf(stderr, "Error opening fifth output file.");
         exit(1);
      };
   
   /****************************     BEGIN FIRST IMAGE    ************************/
   
      index = 0;
      u = -(Pi/2);
   
      while(u <= (Pi/2))
      {
         v = -(Pi);
         while (v <= Pi)
         {
            Su = sin(u); SuAbs = fabs(Su); /* get sin and abs value of sin */
         
            Su2E1 = (sign(Su) * (pow(SuAbs, E1))); /* raise Su to power E */
         
            Sv = sin(v); SvAbs = fabs(Sv); /* get sin and abs value of sin */
            Sv2E2 = (sign(Sv) * (pow(SvAbs, E2))); /* raise Sv to power E */
         
         
            Cu = cos(u); CuAbs = fabs(Cu); /* get cos and abs value of cos */
            Cu2E1 = (sign(Cu) * (pow(CuAbs, E1))); /* raise Cu to power E */
         
            Cv = cos(v); CvAbs = fabs(Cv); /* get cos and abs value of cos */
            Cv2E2 = (sign(Cv) * (pow(CvAbs, E2))); /* raise Cv to power E */
         
            XYZ[index].x_coord = (W * (a1x * (Cu2E1 * Cv2E2)));
            XYZ[index].y_coord = (W * (a2y * (Cu2E1 * Sv2E2)));
            XYZ[index].z_coord = (W * (a3z * Su2E1));
         
            index++;
            v = v + V_STEP;
         }
         u = u + U_STEP;
      }
   
      display_offset = 24;
      X_OFFSET = 0.8;
      Y_OFFSET = 0.7;
      Z_FORWARD = 20.0;
      MAGNIFY = 2700.0;
   
      angle = 2.7925;
      for(k = 0; k < index; k++)
      {
         rotate (Y_AXIS, angle, &XYZ[k].x_coord, &XYZ[k].y_coord, &XYZ[k].z_coord);
      }
      angle = 0.17453;
      for(k = 0; k < index; k++)
      {
         rotate (X_AXIS, angle, &XYZ[k].x_coord, &XYZ[k].y_coord, &XYZ[k].z_coord);
         newX = (XYZ[k].x_coord + X_OFFSET);
         newY = (XYZ[k].y_coord + Y_OFFSET);
         newZ = (XYZ[k].z_coord + Z_FORWARD);
         imageX = (int)(MAGNIFY *(newX/newZ));
         imageY = (int)(MAGNIFY *(newY/newZ));
         first_image[imageY+display_offset][imageX+display_offset] = BRIGHT;
      
      }
   
      for (i = 0; i < 45; i++)
      {
         for (j = 0; j < 256; j++)
         {
            first_image[i][j] = 0;
         }
      }  /* ??fix?? */
   
   /*-----------------------------------------------------------output 1st image */
      display(first_image, &outfp1);
   
   
   /*****************************     END FIRST IMAGE    *************************/
   
   /***************************     BEGIN SECOND IMAGE    ************************/
   
      for (k = 0; k < RANGE; k++)
      {
         clear_xyz(&XYZ[k].x_coord, &XYZ[k].y_coord, &XYZ[k].z_coord);
      }  /* clear */
   
      a3z = (4.0 * a1x);
      index = 0;
      u = -(Pi/2);
   
      while(u <= (Pi/2))
      {
         v = -(Pi);
         while (v <= Pi)
         {
            Su = sin(u); SuAbs = fabs(Su); /* get sin and abs value of sin */
         
            Su2E1 = (sign(Su) * (pow(SuAbs, E1))); /* raise Su to power E */
         
            Sv = sin(v); SvAbs = fabs(Sv); /* get sin and abs value of sin */
            Sv2E2 = (sign(Sv) * (pow(SvAbs, E2))); /* raise Sv to power E */
         
         
            Cu = cos(u); CuAbs = fabs(Cu); /* get cos and abs value of cos */
            Cu2E1 = (sign(Cu) * (pow(CuAbs, E1))); /* raise Cu to power E */
         
            Cv = cos(v); CvAbs = fabs(Cv); /* get cos and abs value of cos */
            Cv2E2 = (sign(Cv) * (pow(CvAbs, E2))); /* raise Cv to power E */
         
            XYZ[index].x_coord = (W * (a1x * (Cu2E1 * Cv2E2)));
            XYZ[index].y_coord = (W * (a2y * (Cu2E1 * Sv2E2)));
            XYZ[index].z_coord = (W * (a3z * Su2E1));
         
            index++;
            v = v + V_STEP;
         }
         u = u + U_STEP;
      }
   
      display_offset = 65;
      X_OFFSET = 0.8;	  		  /* x offset to see 3Dness if image */
      Y_OFFSET = 0.8; 		  /* y offset to see 3Dness if image */
      MAGNIFY = 1500.0;          /* magnify factor */
   
      angle = 0.785398163;
      for(k = 0; k < index; k++)
      {
         rotate (Y_AXIS, angle, &XYZ[k].x_coord, &XYZ[k].y_coord, &XYZ[k].z_coord);
      
         newX = (XYZ[k].x_coord + X_OFFSET);
         newY = (XYZ[k].y_coord + Y_OFFSET);
         newZ = (XYZ[k].z_coord + Z_FORWARD);
         imageX = (int)(MAGNIFY *(newX/newZ));
         imageY = (int)(MAGNIFY *(newY/newZ));
         sec_image[imageY+display_offset][imageX+display_offset] = BRIGHT;
      }
   
      for (i = 0; i < 10; i++)
      {
         for (j = 0; j < 256; j++)
         {
            sec_image[i][j] = 0;
         }
      }  /* ??fix?? */
   
   /*-----------------------------------------------------------output 2nd image */
      display(sec_image, &outfp2);
   
   /*****************************     END SECOND IMAGE    ************************/
   
   
   /***************************     BEGIN THIRD IMAGE    *************************/
   
      for (k = 0; k < RANGE; k++)
      {
         clear_xyz(&XYZ[k].x_coord, &XYZ[k].y_coord, &XYZ[k].z_coord);
      }  /* clear */
      W = 0.5 + W;
      a3z = a2y;
      a1x = (2.0 * a2y);
      index = 0;
      u = -(Pi/2);
   
      while(u <= (Pi/2))
      {
         v = -(Pi);
         while (v <= Pi)
         {
            Su = sin(u); SuAbs = fabs(Su); /* get sin and abs value of sin */
         
            Su2E1 = (sign(Su) * (pow(SuAbs, E1))); /* raise Su to power E */
         
            Sv = sin(v); SvAbs = fabs(Sv); /* get sin and abs value of sin */
            Sv2E2 = (sign(Sv) * (pow(SvAbs, E2))); /* raise Sv to power E */
         
         
            Cu = cos(u); CuAbs = fabs(Cu); /* get cos and abs value of cos */
            Cu2E1 = (sign(Cu) * (pow(CuAbs, E1))); /* raise Cu to power E */
         
            Cv = cos(v); CvAbs = fabs(Cv); /* get cos and abs value of cos */
            Cv2E2 = (sign(Cv) * (pow(CvAbs, E2))); /* raise Cv to power E */
         
            XYZ[index].x_coord = (W * (a1x * (Cu2E1 * Cv2E2)));
            XYZ[index].y_coord = (W * (a2y * (Cu2E1 * Sv2E2)));
            XYZ[index].z_coord = (W * (a3z * Su2E1));
         
            index++;
            v = v + V_STEP;
         }
         u = u + U_STEP;
      }
   
      display_offset = 40;
      X_OFFSET = 1.8;	  		  /* x offset to see 3Dness if image */
      Y_OFFSET = 1.5;	  		  /* y offset to see 3Dness if image */
      MAGNIFY = 1200.0;          /* magnify factor */
   
      t1x = 1.0;
      t2y = 1.0;
      b1 = 0;       /* no bending */
      b2 = 0;
      b3 = 0;
   
      for(k = 0; k < index; k++)
      {
         newX = XYZ[k].x_coord;
         newY = XYZ[k].y_coord;
         newZ = XYZ[k].z_coord;
         XYZ[k].x_coord = ((((t1x * newZ) / (a3z * W)) + 1) * newX)
                          + (b1 * cos(((newZ * b2) * Pi * b3) / (a3z * W)));
         XYZ[k].y_coord = ((((t2y * newZ) / (a3z * W)) + 1) * newY);
         XYZ[k].z_coord = newZ;
      }
      angle = 4.6123;
      for(k = 0; k < index; k++)
      {
         rotate (Y_AXIS, angle, &XYZ[k].x_coord, &XYZ[k].y_coord, &XYZ[k].z_coord);
      }
      angle = 0.0;
      for(k = 0; k < index; k++)
      {
         rotate (Z_AXIS, angle, &XYZ[k].x_coord, &XYZ[k].y_coord, &XYZ[k].z_coord);
      }
      for(k = 0; k < index; k++)
      {
         newX = (XYZ[k].x_coord + X_OFFSET);
         newY = (XYZ[k].y_coord + Y_OFFSET);
         newZ = (XYZ[k].z_coord + Z_FORWARD);
         imageX = (int)(MAGNIFY *(newX/newZ));
         imageY = (int)(MAGNIFY *(newY/newZ));
         thr_image[imageY+display_offset][imageX+display_offset] = BRIGHT;
      }
   
      for (i = 0; i < 10; i++)
      {
         for (j = 0; j < 256; j++)
         {
            thr_image[i][j] = 0;
         }
      }  /* ??fix?? */
   
   /*-----------------------------------------------------------output 3Rd image */
      display(thr_image, &outfp3);
   
   /*****************************     END THIRD IMAGE    *************************/
   
   
   /***************************     BEGIN FORTH IMAGE    *************************/
   
      for (k = 0; k < RANGE; k++)
      {
         clear_xyz(&XYZ[k].x_coord, &XYZ[k].y_coord, &XYZ[k].z_coord);
      }  /* clear */
   
      a1x = a2y;
      a3z = (3.0 * a1x);
      index = 0;
      u = -(Pi/2);
   
      while(u <= (Pi/2))
      {
         v = -(Pi);
         while (v <= Pi)
         {
            Su = sin(u); SuAbs = fabs(Su); /* get sin and abs value of sin */
         
            Su2E1 = (sign(Su) * (pow(SuAbs, E1))); /* raise Su to power E */
         
            Sv = sin(v); SvAbs = fabs(Sv); /* get sin and abs value of sin */
            Sv2E2 = (sign(Sv) * (pow(SvAbs, E2))); /* raise Sv to power E */
         
         
            Cu = cos(u); CuAbs = fabs(Cu); /* get cos and abs value of cos */
            Cu2E1 = (sign(Cu) * (pow(CuAbs, E1))); /* raise Cu to power E */
         
            Cv = cos(v); CvAbs = fabs(Cv); /* get cos and abs value of cos */
            Cv2E2 = (sign(Cv) * (pow(CvAbs, E2))); /* raise Cv to power E */
         
            XYZ[index].x_coord = (W * (a1x * (Cu2E1 * Cv2E2)));
            XYZ[index].y_coord = (W * (a2y * (Cu2E1 * Sv2E2)));
            XYZ[index].z_coord = (W * (a3z * Su2E1));
         
            index++;
            v = v + V_STEP;
         }
         u = u + U_STEP;
      }
   
      display_offset = 30;
      X_OFFSET = 2.5;  		  /* x offset to see 3Dness if image */
      Y_OFFSET = 2.0;	  		  /* y offset to see 3Dness if image */
      MAGNIFY = 850.0;          /* magnify factor */
   
      t1x = 0.0;                /* no TAPERING */
      t2y = 0.0;
      b1 = 1.5;                 /* magnitude of the bend */
      b2 = 1.0;					  /* location of the bend */
      b3 = 1.0;					  /* region of influence */
   
      for(k = 0; k < index; k++)
      {
         newX = XYZ[k].x_coord;
         newY = XYZ[k].y_coord;
         newZ = XYZ[k].z_coord;
         XYZ[k].x_coord = ((((t1x * newZ) / (a3z * W)) + 1) * newX)
                          + (b1 * cos(((newZ * b2) * Pi * b3) / (a3z * W)));
         XYZ[k].y_coord = ((((t2y * newZ) / (a3z * W)) + 1) * newY);
         XYZ[k].z_coord = newZ;
      }
      angle = 1.5;
      for(k = 0; k < index; k++)
      {
         rotate (Z_AXIS, angle, &XYZ[k].x_coord, &XYZ[k].y_coord, &XYZ[k].z_coord);
      }
      angle = 0.657;
      for(k = 0; k < index; k++)
      {
         rotate (Y_AXIS, angle, &XYZ[k].x_coord, &XYZ[k].y_coord, &XYZ[k].z_coord);
      }
      angle = 0.0;
      for(k = 0; k < index; k++)
      {
         rotate (Z_AXIS, angle, &XYZ[k].x_coord, &XYZ[k].y_coord, &XYZ[k].z_coord);
      }
      for(k = 0; k < index; k++)
      {
         newX = (XYZ[k].x_coord + X_OFFSET);
         newY = (XYZ[k].y_coord + Y_OFFSET);
         newZ = (XYZ[k].z_coord + Z_FORWARD);
         imageX = (int)(MAGNIFY *(newX/newZ));
         imageY = (int)(MAGNIFY *(newY/newZ));
         forth_image[imageY+display_offset][imageX+display_offset] = BRIGHT;
      }
   
      for (i = 0; i < 10; i++)
      {
         for (j = 0; j < 256; j++)
         {
            forth_image[i][j] = 0;
         }
      }  /* ??fix?? */
   
   /*-----------------------------------------------------------output 4th image */
      display(forth_image, &outfp4);
   
   /*****************************     END FORTH IMAGE    *************************/
   
   
   /***************************     BEGIN FIFTH IMAGE    *************************/
      for (k = 0; k < RANGE; k++)
      {
         clear_xyz(&XYZ[k].x_coord, &XYZ[k].y_coord, &XYZ[k].z_coord);
      }  /* clear */
   
      a1x = a2y;
      a3z = (4.0 * a1x);
      index = 0;
      u = -(Pi/2);
   
      while(u <= (Pi/2))
      {
         v = -(Pi);
         while (v <= Pi)
         {
            Su = sin(u); SuAbs = fabs(Su); /* get sin and abs value of sin */
         
            Su2E1 = (sign(Su) * (pow(SuAbs, E1))); /* raise Su to power E */
         
            Sv = sin(v); SvAbs = fabs(Sv); /* get sin and abs value of sin */
            Sv2E2 = (sign(Sv) * (pow(SvAbs, E2))); /* raise Sv to power E */
         
         
            Cu = cos(u); CuAbs = fabs(Cu); /* get cos and abs value of cos */
            Cu2E1 = (sign(Cu) * (pow(CuAbs, E1))); /* raise Cu to power E */
         
            Cv = cos(v); CvAbs = fabs(Cv); /* get cos and abs value of cos */
            Cv2E2 = (sign(Cv) * (pow(CvAbs, E2))); /* raise Cv to power E */
         
            XYZ[index].x_coord = (W * (a1x * (Cu2E1 * Cv2E2)));
            XYZ[index].y_coord = (W * (a2y * (Cu2E1 * Sv2E2)));
            XYZ[index].z_coord = (W * (a3z * Su2E1));
         
            index++;
            v = v + V_STEP;
         }
         u = u + U_STEP;
      }
      for(k = 0; k < index; k++)
      {
         newX = XYZ[k].x_coord;
         newY = XYZ[k].y_coord;
         newZ = XYZ[k].z_coord;
         XYZ[k].x_coord = ((((t1x * newZ) / (a3z * W)) + 1) * newX)
                          + (b1 * cos(((newZ * b2) * Pi * b3) / (a3z * W)));
         XYZ[k].y_coord = ((((t2y * newZ) / (a3z * W)) + 1) * newY);
         XYZ[k].z_coord = newZ;
      }
   
      display_offset = 35;
      X_OFFSET = 2.1;  		  /* x offset to see 3Dness if image */
      Y_OFFSET = 2.0;	  		  /* y offset to see 3Dness if image */
      MAGNIFY = 700.0;          /* magnify factor */
   
   
      t1x = 0.0;                /* no TAPERING */
      t2y = 0.0;
      b1 = 0.3;                 /* magnitude of the bend */
      b2 = 1.0;					  /* location of the bend */
      b3 = 1.5;					  /* region of influence */
      TAU = 1.0;
   
   
      for(k = 0; k < index; k++)
      {
         newX = XYZ[k].x_coord;
         newY = XYZ[k].y_coord;
         newZ = XYZ[k].z_coord;
         P = ((((t1x * newZ) / (a3z * W)) + 1) * newX)
             + (b1 * cos(((newZ * b2) * Pi * b3) / (a3z * W)));
         Q = ((((t2y * newZ) / (a3z * W)) + 1) * newY);
         PHI = ((newZ * Pi * TAU) / (a3z * W));
         XYZ[k].x_coord = ((P * cos(PHI)) - (Q * sin(PHI)));
         XYZ[k].y_coord = ((P * sin(PHI)) + (Q * cos(PHI)));
         XYZ[k].z_coord = newZ;
      }
      angle =  0.65;
      for(k = 0; k < index; k++)
      {
         rotate (X_AXIS, angle, &XYZ[k].x_coord, &XYZ[k].y_coord, &XYZ[k].z_coord);
      }
      angle = 0.30;
      for(k = 0; k < index; k++)
      {
         rotate (Y_AXIS, angle, &XYZ[k].x_coord, &XYZ[k].y_coord, &XYZ[k].z_coord);
      }
      angle = 1.1;
      for(k = 0; k < index; k++)
      {
         rotate (Z_AXIS, angle, &XYZ[k].x_coord, &XYZ[k].y_coord, &XYZ[k].z_coord);
      }
      for(k = 0; k < index; k++)
      {
         newX = (XYZ[k].x_coord + X_OFFSET);
         newY = (XYZ[k].y_coord + Y_OFFSET);
         newZ = (XYZ[k].z_coord + Z_FORWARD);
         imageX = (int)(MAGNIFY *(newX/newZ));
         imageY = (int)(MAGNIFY *(newY/newZ));
         fifth_image[imageY+display_offset][imageX+display_offset] = BRIGHT;
      }
   
      for (i = 0; i < 45; i++)
      {
         for (j = 0; j < 256; j++)
         {
            fifth_image[i][j] = 0;
         }
      }  /* ??fix?? */
   
   /*-----------------------------------------------------------output 5th image */
      display(fifth_image, &outfp5);
   
   /*****************************     END FIFTH IMAGE    *************************/
   
   
   /*--------------------------------------------------------------  close files */
      fclose(outfp1);
      fclose(outfp2);
      fclose(outfp3);
      fclose(outfp4);
      fclose(outfp5);
   
      return EXIT_SUCCESS;
   }  /* end main */

   double sign (double SinCos_value)
   /* takes the value for a double and returns 1 if positive or -1 if negative */
   {
      double neg = -1.0,
      pos = 1.0;
   
      if(SinCos_value < 0)
         return neg;
      else
         return pos;
   } /* end sign */

   void display (int outpic[Y_DIMENSION][X_DIMENSION], FILE **fp)
   /* takes an array of int and a file pointer and outputs it to the file */
   {
      int iy, jx;
   
      for (iy = 0; iy < Y_DIMENSION; iy++)
      {
         for (jx = 0; jx < X_DIMENSION; jx++)
         {
            fprintf(*fp,"%c",(char)outpic[iy][jx]);
         }
      }
   } /* end display */

   void clear_xyz(double *x, double *y, double *z)
   {
      *x = ZERO;
      *y = ZERO;
      *z = ZERO;
   }

   void rotate ( int axis, double ang, double *Rx, double *Ry, double *Rz)
   {
      double x, y, z;
   
      x = *Rx; y = *Ry; z = *Rz;
      if (axis == 1)
      {
         *Rx = (1.0 * x);
         *Ry = (cos(ang) * y) - (sin(ang) * z);
         *Rz = (sin(ang) * y) + (cos(ang) * z);
      }
      else if (axis == 2)
      {
         *Rx = (cos(ang) * x) + (sin(ang) * z);
         *Ry = (1.0 * y);
         *Rz = (cos(ang) * z) - (sin(ang) * x);
      }
      else if (axis == 3)
      {
         *Rx = (cos(ang) * x) - (sin(ang) * y);
         *Ry = (sin(ang) * x) + (cos(ang) * y);
         *Rz = (1.0 * z);
      }
   } /* end rotate */

/* end main a4.c */

