/* PASS2.C
/*
/*   SSSS    OOOO   U    U  RRRRR    CCCC   EEEEEE
/*  S       O    O  U    U  R    R  C    C  E
/*  S       O    O  U    U  R    R  C       E
/*   SSSS   O    O  U    U  RRRRR   C       EEE
/*       S  O    O  U    U  R  R    C       R
/*       S  O    O  U    U  R   R   C    C  R
/*   SSSS    OOOO    UUUU   R    R   CCCC   REEEEE
/*****************************************************************************\
			   Fire, Earth, and Wind
	   (a.k.a. Hai Nguyen, Bettina McCormick  & Rob Konczynski)

	      April 1998, COP 3402 - Systems, Assignment #3

		   Pass 2  - Due Date: April 27, 1998

	Version 3.141592653589793238884- last updated  April 25, 1998

 *-------------------------- PASS 2 MAIN PROGRAM  ----------------------------*

\******************************************************************************/	



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>

/*************************    # D E F I N E S    ******************************/

/*-------------------------    OPERATION TABLE    ----------------------------*/
#define OP_LENGTH     6       /* maximum number of chars in operation */
#define OT_SIZE     100       /* max number of entries in Operation Table */


/*--------------------------     SYMBOL TABLE    -----------------------------*/
#define REG_TOTAL     9       /* # of registers to load into symtab */
#define HT_SIZE      59       /* # of entries in the symtab_hash table */
#define STEP_SIZE    29       /* value to be added to the symtab_hash value
                    		for resolving symtab_hash collisions */




/*-----------------------------     PASS2    ---------------------------------*/
#define NAME_LENGTH  10       /* maximum number of characters in name */
#define OPND_LENGTH  50       /* maximum number of chars in operand */
#define INP_LENGTH   70       /* max chars in an input assembly line */
#define OBJ_LENGTH    4       /* max length of object code in bytes */
#define PC_LIMIT   1023       /* if displacements are greater than this value
                                 then we switch to base relative*/

/*-------------------------------   MAIN   -----------------------------------*/
/* #define DEBUG */           /* Used for error checking */
#define TRUE         1        /* Boolean value which indicates true */
#define FALSE        0        /* Boolean value which indicates false */
#define NADA        -1        /* Value used for empty SYMTAB slots */
#define RELOCATABLE 'R'       /* Indicates a Relocatable address */
#define ABSOLUTE    'A'       /* Indicates an Absolute address */
#define REGISTER    'N'       /* Indicates an Register address */
#define BLANK       ' '       /* Defines a blank space (duh!) */
#define DELIM       " \t\n\0" /* Defines the different symbols which delimit
                                  strings */

/*************************    S T R U C T U R E S    **************************/

/*-----------------------  OPERATION TABLE STRUCTS   -------------------------*/
   struct ot_entry {          /* an entry in Operation Table */
      char ot_mnemonic[OP_LENGTH + 1];
      int  ot_format;  /* 0 (assembler directive), 1,2,3 (formats 1,2,3/4) */
      char ot_temp[OP_LENGTH + 1];
      unsigned char ot_opcode;  /* operation code */
   };

   struct ot_entry optbl[OT_SIZE];  /* Operation Table; Operation Table is
                           		sorted with a binary search*/

   int     ot_count;       /*number of entries in Operation Table */
/*--------------------  END OPERATION TABLE STRUCTS   ------------------------*/

/*--------------------------  SYMBOL TABLE STRUCTS   -------------------------*/
   struct st_entry {          	/* an entry in the Symbol Table */
      char st_name[NAME_LENGTH + 1];
      char st_type;           	/* R(elocatable) or A(bsolute) */
      int  st_value;       	/* value of the label */
   };

   struct st_entry symtbl[HT_SIZE]; /* Symbol Table; Symbol Table is maintained
                   			using hashing */
/*----------------------  END SYMBOL TABLE STRUCTS   -------------------------*/


/**********************      D E C L A R A T I O N S    ***********************/

/*-----------------------------   MAIN DECLARATIONS  ----------------------*/
   FILE *ot_fptr,     	/* File pointer used to open the operation table input*/
   *st_fptr,     	/* File pointer used to open the symbol table input*/
   *temp_fptr,   	/* File pointer used to open the temporary file*/
   *in_fptr,     	/* File pointer used to open the assembly program */
   *out_fptr,    	/* File pointer used to open the output file */
   *fopen();     	/* function for opening files */

   typedef unsigned char byte; /* to refer to char as byte */

   int  locctr;       /* location counter; this is incremented by the
                   	appropriate value for each instruction;
                	GLOBAL VARIABLE */

   int  lineno;       /* line number; this is incremented by 1 for each input
                   	assembly language line;
                	GLOBAL VARIABLE */

   int basereg;

   byte objcode[OBJ_LENGTH+1];  /* object code */

   int set[8]={0};

   char HEXDIG[16] = {'0','1','2','3','4','5','6','7',
                     '8','9','A','B','C','D','E','F' };

   /* these are different values for a byte when a hex digit is placed in it's
   rightmost half-byte, that is the four left most bits are zippo--this is in
   octal form */
   char RMHBHEX[16]= {'\000', '\001','\002','\003','\004','\005','\006','\007',
                     '\010', '\011','\012','\013','\014','\015','\016','\017' };

   int spaces[5]= {14, 13, 10, 7, 4};
   /*depending on what format you have, this array will tell you how many spaces
   between objcode in hex and objcode in binary you need to move over. 
	Example:   format 2's need 11 spaces */


/*----------------------------   END MAIN DECLARATIONS  ----------------------*/


/************************    P R O T O T Y P E S     **************************/

/*-------------------------    OPERATION TABLE PROTOTYPES   ------------------*/
void optab_init(struct ot_entry *ot);
   /* Initializes the operation table. */

void optab_load(FILE **, struct ot_entry *, int *);
   /* This routine is used by the init_pass2 routine.
      It loads the optab. */

void optab_write(FILE **, struct ot_entry *, int);
   /* This routine is used by the init_pass2 routine.
      It writes the optab to the ouput file. */
/*---------------------  END OPERATION TABLE PROTOTYPES   ------------------*/

/*------------------------  SYMBOL TABLE PROTOTYPES   ----------------------*/

void symtab_init(struct st_entry *);
   /* Initializes the symbol table. */

int symtab_hash(char *);
   /* Takes a pointer to a char array and returns the sum of the first and
      last capital letters (in the char array) - 2, where A = 1 and Z = 26.
		For example CONE returns 3 + 5 - 2 = 6. */

int symtab_search(char *, struct st_entry *, int *);
   /* Takes the label, symtab, and a pointer to the original hash index;
	searches the symtab for label - if found returns TRUE;
        else returns false; the index becomes the latest place to insert. */

void symtab_insert(char *, char , struct st_entry *, int , int);
   /* Pre-condition: symtab is not full and label is not found;
	Post-condition: struct of label name, type and value is inserted
	into the symtab.
	Takes the label, type, symtab and the index (already hashed and
	searched) to the empty slot. */

void symtab_file_load(struct st_entry *st, FILE **fp);
   /* Loads the symbol table witn the info from the symtab file. */

void symtab_register_load(struct st_entry *st);
   /* Loads the symbol table with the register labels and values */

void symtab_write(struct st_entry *, FILE **);
   /* Writes the symbol table to the output file. */

/*------------------------ END SYMBOL TABLE PROTOTYPES   ---------------------*/

/*---------------------------   MAIN  PROTOTYPES   ---------------------------*/
void init_pass2(int, char **);
   /* This routine is used by the main routine.
	It takes the number of arguements and the argv array of argurments.
	It initializes the global variables locctr and lineno.
	It opens the three files (sets ot_fptr, in_fptr, out_fptr).
	It loads optab (using ot_fptr) and sorts optab.
	It writes the sorted optab to output file.
	It initializes symtab.*/

void get_tokens1(char line[INP_LENGTH +1],char lbl[NAME_LENGTH +1],
		 char  opr[OP_LENGTH + 1],  char ond[OPND_LENGTH + 1],
 		 int *op_ind, int *len);
   /* This routine is used by pass2;  takes the line, label, operator, and
	operand - parses the line and fills the three variables respectively. */

void fmt0(FILE **fp, char *, char *, char *, struct st_entry *);
   /* This routine is used by pass2 to process format 0 (assembler 
    	directives).  */

void fmt1(FILE **, struct ot_entry *, int);
   /* This routine is used by pass2 to process format 1. */

void fmt2(FILE **fp, struct ot_entry *ot, int opidx, char  opr[OP_LENGTH + 1], 
	  char ond[OPND_LENGTH + 1], struct st_entry *st);
   /* This routine is used by pass2 to process format 2.
	takes the file pointer to the temp file, the optab, op index,
	operand to be parsed, and the symtab;
	creates object code for the instruction and prints it to
	the temp file in hex and binary.  */

/*---------------------- FORMAT 3 PROTOTYPES   ---------------------*/
int convert_to_hex(int number, char *hex_2, char *hex_1, char *hex_0);
   /* Takes in an integer and changes its 6 hex digits into
	charcters that store 2 hex digits at one time.
	Basically convert a number into 3 bytes of info */

int symtab_get(struct st_entry *Symbol_Table, struct st_entry *Return_record,
			 char label_to_find[]);
   /* Finds a label in the symbol table passed in and then
	passes it's data back through a structure called
	Return_record;  If data is NOT Found return false. */

int scan_string_for(char character, char string[]);
   /* Accepts a string and reports back the index of the found
	character.  The character to look for is indicated by
	character.  If data is NOT Found return false. */

int parse_string_for_fmt3(char operand[OPND_LENGTH], char *addressing_scheme,
		  char left_side[NAME_LENGTH], char right_side[NAME_LENGTH]);
   /* Accepts a string which is the operand, a character pointer
	and two strings left_side, and right_side.  The function
	will parse the operand into two fields which is delimited
	by a ','.   The function will return the left_side
	string, and the right side_string.  If the right side
	does not exist then the first char of the right side
	will be set to the EOS(End of String) Character '\0' */

int object_code_generate_displacement(byte object_code[OBJ_LENGTH],
		int location_counter, int left_value, int base_reg);
   /* Generates the displacement of an relocatable address.
	The function accepts an array of object code which is
	defined as bytes.  It will adjust the last 3 nibbles
	also known as half bytes to contain the calculated
	displacemnet based on the location counter.  The base
	register variable will be used if displacement is
	beyond the Program Counter Range. */

void fmt3(FILE **Output, 	byte object_code[OBJ_LENGTH],
		struct ot_entry operation_table[], char operand[OPND_LENGTH],
		int  operation_index, struct st_entry *Symbol_Table,
		int location_counter, int lineno,  int base_reg);
   /* To Process Format 3 instructions as indicated by the
	SIC / SIC XE Virtual Computer based on the methods
	explained in the following cited book:
		"SYSTEM SOFTWARE an Introduction to Systems Programming"
		Author : Leland L. Beck
		Edition: 3rd
		Publisher: ADDISON-WESLEY Longman, INC
		ISBN : 0-201-42300-6
		Dewi Decimal Assignment : QA76.66B42  1997
	Format 3 Instructions are instructions that allow
	Program Counter, OR Base Relative Addressing.  They
	also allow Indirect, Direct and Simple Addressing.
	The above mentioned schemes are deterimend in this
	format. */

/*-------------------------- END FORMAT 3 PROTOTYPES  ------------------------*/

void fmt4(FILE **fp, struct ot_entry *ot, int, struct st_entry *st,char *, char *);
   /* This routine is used by pass2 to process format 4. */

void copy_temp_file(FILE **temp, FILE **out);
   /* This routine is used by the main routine after all the input   */
   /* have been processed.  This routine copies the temporary output */
   /* file (temp_fptr) into the main output file (out_fptr).         */

void pass2(void);
   /* This routine is used by the main routine after all the input   */
   /* have been processed.  This routine copies the temporary output */
   /* file (temp_fptr) into the main output file (out_fptr).         */


void two_hex_to_byte(char, char, char *);
   /* takes two chars, which are really hex digits and stuffs 'em together
       	into the char *       */

void byte_to_hex(FILE**, char);
   /* takes a byte as an argument and prints out the hexadecimal form of that
	byte */

void decimal_to_hex (int, char *);
   /* Takes a decimal number and coverts it into it's hexadecimal form--entering
	it into word_array */

/*--------------------------  BIT STUFF PROTOTYPES   -------------------------*/
char set_bits (int, int);
   /*  returns a char with the bits set for the two hex digits passed;
	 the first hex value is R2, the second is R1
	 The Bettina-brute-force way
	 - my hats off to Hai and his slick "shift the bits"!  */

void flip_the_bit (int, char *);
   /*  prints bits of a char to a file */

void bit_print (FILE **, char );
   /* This function, like--it umm prints the bit representation of the
	value sent. USED FOR CHARS ONLY!!!*/

/*------------------------  END MAIN  PROTOTYPES   --------------------------*/


/***************************        M A I N       ****************************/
int main (int argc, char **argv)
{
	init_pass2(argc, argv);
   
	pass2();

	copy_temp_file(&temp_fptr, &out_fptr);

	printf("\nPass2 has successfully processed your input, ");
        printf(" \"%s\".",  argv[3]);
        printf("\n\nThe output and other vital ");
        printf("data have been directed");
        printf("\nto your indicated ouput file, \"%s\".\n", argv[4]);
   
/*--------------------     close files     -----------------------*/
      	fclose(ot_fptr);
        fclose(st_fptr);
	fclose(temp_fptr);
     	fclose(in_fptr);
      	fclose(out_fptr);
   
      	return EXIT_SUCCESS;
   
} /* end MAIN */
/***************************   END  M A I N       ****************************/




/***************************   F U N C T I O N S  ****************************/
/*---------------------    OPERATION TABLE FUNCTIONS  ----------------------*/
/* Here Lies a Link to optbpart.c  The file with the operation table functions*/
#include"optbpart.c"
/*-------------------- END OPERATION TABLE FUNCTIONS  ----------------------*/


/*--------------------     SYMBOL TABLE  FUNCTIONS  ------------------------*/
/* Here Lies a Link to smtbpart.c  The file with the symbol table functions */
#include"smtbpart.c"
/*--------------------  END SYMBOL TABLE  FUNCTIONS  -----------------------*/

/*-----------------------------   MAIN  FUNCTIONS  -------------------------*/
/********************************************************************/
void init_pass2(int argc, char *argv[])
   /* This routine is used by the main routine.
	It takes the numver of arguements and the argv array of argurments.
	It initializes the global variables locctr and lineno.
	It opens the three files (sets ot_fptr, in_fptr, out_fptr).
	It loads optab (using ot_fptr) and sorts optab.
	It writes the sorted optab to output file.
	It initializes symtab.
	It initializes errque.*/
{ /* begin init_pass2 */
 /*---------------     initialize variables     ------------------*/
	int i;

	locctr = 0;
	lineno = 0;
	ot_count = 0;
	basereg = 0;

	for (i = 0; i < 4; i++)
		objcode[i] = 0;
/*-----------------  end  initialize variables     --------------*/

/*--------------------     open files     -----------------------*/
	if (argc < 5)
	{
		 printf("You did not enter the correct number of file arguements.");
		 exit(1);
	}; /* end if */
	if((ot_fptr=fopen(argv[1], "r")) == NULL) /* opens argv[1] file */
	{
		fprintf(stderr, "Error opening optab file.");
		exit(1);
	};
	if((st_fptr=fopen(argv[2], "r")) == NULL) /* opens argv[2] file */
	{
		fprintf(stderr, "Error opening symtab file.");
		exit(1);
	};
	if((in_fptr=fopen(argv[3], "r")) == NULL)   /* opens argv[3] file */
	{
		fprintf(stderr, "Error opening input file.");
		exit(1);
	};
	if( ( out_fptr=fopen(argv[4], "w")) == NULL)   /* opens argv[4] file */
	{
		fprintf(stderr, "Error opening output file.");
		exit(1);
	};
	if((temp_fptr=fopen("temp.dat", "w")) == NULL) /* opens temp file */
	{
		fprintf(stderr, "Error opening temp file.");
		exit(1);
	};
/*---------------------  end open files     -----------------------*/

/*---------------     create the Op table      --------------------*/
	optab_init(optbl);
	optab_load(&ot_fptr, optbl, &ot_count);
/*-----------------------------------------------------------------*/
/*---------------     write the Op table      ---------------------*/
	optab_write(&out_fptr, optbl, ot_count);
/*-----------------------------------------------------------------*/

/*---------------     create the Sym table      -------------------*/
	symtab_init(symtbl);
	symtab_register_load(symtbl);
	symtab_file_load(symtbl, &st_fptr);
/*-----------------------------------------------------------------*/
/*---------------     write the Sym table      --------------------*/
	symtab_write(symtbl, &out_fptr);
/*-----------------------------------------------------------------*/
} /* end init_pass2 */
  /* submitted by Mother Earth */

/********************************************************************/
/******************************************************************************/
/* Here Lies a Link to tok1part.c  The file with get_tokens function */
/* this get tokens works - if we never have to use to other one then we'll cut
	the other one out of fmt0part.c */
#include"tok1part.c"
/******************************************************************************/
/******************************************************************************/
/* Here Lies a Link to fmt0part.c  The file with get_tokens & fmt0 functions */
/* the get_tokens in this file is not used currently - it blew up on the first
	line that wasn't a comment - so I cut and pasted the get_tokens from the
	version that rob sent me and it works great so far */
#include"fmt0part.c"
/******************************************************************************/


/******************************************************************************/
/* Here Lies a Link to fm12part.c  The file with the fmt1 & 2 functions */
#include"fm12part.c"
/******************************************************************************/
/******************************************************************************/
/* Here Lies a Link to FMT3PART.C  The file with the fmt3 functions */
#include"fmt3part.c"
/******************************************************************************/

/******************************************************************************/
/* Here Lies a Link to fmt4part.c  The file with the fmt4 & COPYFILE functions*/
#include"fmt4part.c"
/******************************************************************************/
/********************************************************************/
void pass2()
/* This routine is used by the main. It performs Pass1 of the assembler. */
{
     	char inp_line[INP_LENGTH + 1]; /* input assembly line */
      	char new_oper[OP_LENGTH + 1];  /* Temporary space */
      	char label[NAME_LENGTH +1];    /* label in the input */
      	char oper[OP_LENGTH + 1];      /* operation in the input */
      	char opnd[OPND_LENGTH + 1];    /* operand in the input; */
			/*this will contain everything after the
			  operation in the input*/

	int i;
	int op_ind;
	int len;


	inp_line[0]='\0';
        new_oper[0]='\0';
        label[0]='\0';
	oper[0]='\0';
	opnd[0]='\0';

	fprintf(out_fptr,
	"\n\nLine   Loc   Source Statement              Op Index   Length\n");
	fprintf(temp_fptr,"Line   Objcode in Hex  Objcode on Binary\n");
				 /* Absorbs first & second line */
	while((fgets(inp_line, INP_LENGTH, in_fptr) != NULL) &&
		(strcmp(inp_line,
		"Loc   Source Statement               Op Index   Length\n")!= 0))
	{ };


	while (fgets(&inp_line[0], INP_LENGTH, in_fptr) != NULL)
	{
		if (inp_line[0] != '\n')
		{
			 lineno++;
			 fprintf(out_fptr, "%3d    %s",lineno, inp_line);
		}
	  get_tokens1(inp_line, &label[0], &oper[0], &opnd[0], &op_ind, &len);
	  for (i = 0; i < 4; i++)
		 {
			 objcode[i] = 0;
		 }
	  if(inp_line[6] != '.')
	  {
		switch (optbl[op_ind].ot_format)
		{
		case 0:		fmt0(&temp_fptr,label, oper, opnd, symtbl);
			break;
		case 1:
			  fmt1(&temp_fptr, optbl, op_ind);
			  break;
		case 2:
			  fmt2(&temp_fptr, optbl, op_ind, oper, opnd, symtbl);
			  break;
		case 3:
			  if (oper[0] != '+')
			  {
				 fmt3(&temp_fptr, objcode, optbl, opnd, op_ind, 
					symtbl, locctr, lineno, basereg);
			  }
			  else  if (oper[0] == '+')
			  {
				 fmt4(&temp_fptr, optbl, op_ind, symtbl, label, 
					opnd);
			  }
			  break;
		} /*  End switch*/
		   }/* End Else */
	}/* End while fgets*/
} /*end pass2 */


/******************************************************************************/
/* Here Lies a Link to bitspart.c  The file with the bit stuff functions */
#include"bitspart.c"
/******************************************************************************/

/******************************************************************************/
/* Here Lies a Link to hexhpart.c  The file with Hai's hex functions */
#include"hexhpart.c"
/******************************************************************************/


/*----------------------------  END MAIN  FUNCTIONS  -------------------------*/




