/* SMTBPART.C
/*
/*   SSSS    OOOO   U    U  RRRRR    CCCC   EEEEEE
/*  S       O    O  U    U  R    R  C    C  E
/*  S       O    O  U    U  R    R  C       E
/*   SSSS   O    O  U    U  RRRRR   C       EEE
/*       S  O    O  U    U  R  R    C       R
/*       S  O    O  U    U  R   R   C    C  R
/*   SSSS    OOOO    UUUU   R    R   CCCC   REEEEE
/*****************************************************************************\
			   Fire, Earth, and Wind
	   (a.k.a. Hai Nguyen, Bettina McCormick  & Rob Konczynski, Jr.)

	      April 1998, COP 3402 - Systems, Assignment #3

		   Pass 2  - Due Date: April 27, 1998

 *----------------------     SYMBOL TABLE  FUNCTIONS  ------------------------*

\******************************************************************************/	


/*-----------------------  S Y M T A B _ I N I T   ----------------------------\

 Purpose: Initializes the symbol table.

 Pre-Condition : <None>

 Post-Condition: Symbol table passed is initialized

 Return: void

\-----------------------------------------------------------------------------*/

void symtab_init(struct st_entry *st)
{
	int i;

	for (i=0; i< HT_SIZE; i++)
	{
		st[i].st_name[0]='\0';
		st[i].st_type='Z';
		st[i].st_value=NADA;
	}
} /* end symtab_init */

/*-----------------------  S Y M T A B _ H A S H   ----------------------------\

 Purpose: Takes a pointer to a char array(the label to be hashed) and performs
	  the following hash function:
	  the sum of the first and last capital letters (in the label) - 2,
	  where A = 1 and Z = 26.
	  For example CONE returns 3 + 5 - 2 = 6.

 Pre-Condition : <none> Follow Parameters

 Post-Condition: initial hash index for label is calculated

 Return: the in	initial hash index

\-----------------------------------------------------------------------------*/
int symtab_hash(char *lbl_ptr)
{
	char first_cap = 0;
	char last_cap = 0;

	while (*lbl_ptr != '\0')
	{
		if (*lbl_ptr >= 'A' && *lbl_ptr <= 'Z')
		{
			if (!first_cap)
			{
				first_cap = last_cap = *lbl_ptr - 'A' + 1;
			}
			else
			{
				last_cap = *lbl_ptr - 'A' + 1;
			}
		}
		lbl_ptr++;
	}
	return (first_cap + last_cap) - 2;
} /* end symtab_hash() */

/*-----------------------  S Y M T A B _ S E A R C H  -------------------------\

 Purpose: Takes the label, symtab, and a pointer to the original hash index;
	  searches the symtab for label

 Pre-Condition : hash index has been calculated

 Post-Condition: The symbol table is searched - the hash index is changed if
	  	 there is a collision and the latest index is returned.
		 Program is halted if collision search overflows the size
		 of the hash table.

 Return: if the label is found returns TRUE (anything not zero);
	 else returns FALSE; the index becomes the latest place to insert.

\-----------------------------------------------------------------------------*/
int symtab_search(char lbl[NAME_LENGTH + 1], struct st_entry *st, int *sidx)
{
	int found_flag= FALSE;
	int empty_flag= FALSE;
	int condition;
	int c = 0;

	while (!(empty_flag) && !(found_flag) && (c < HT_SIZE))
	{
		condition = strcmp ((st + *sidx)->st_name, lbl);
		c++;
		if (condition == 0)
		{
			 found_flag=TRUE;
		}
		else if (*(st + *sidx)->st_name == '\0')
		{
			 empty_flag= TRUE;
		}
		else
		{
			*sidx = (*sidx + STEP_SIZE) % HT_SIZE;
		}
	}
	if ((c >= HT_SIZE) && (found_flag == FALSE))
	{
		fprintf(stderr, "Error - Symbol Table Overflow");
		exit(1);
	}
	return found_flag;
} /*end search */

/*-----------------------  S Y M T A B _ I N S E R T  -------------------------\

 Purpose: To insert the label, label type, and label value in the symbol table.

 Pre-Condition : symtab is not full and label is not found, hash index is
		 latest place available to insert
		 - accomplisted if false value is returned from symtab_search;

 Post-Condition: struct of label name, type and
		 value is inserted into the symtab

 Return: void

\-----------------------------------------------------------------------------*/
void symtab_insert(char lbl[NAME_LENGTH + 1], char lbl_type,
                   struct st_entry *st, int sidx, int val)
{
	strcpy(st[sidx].st_name, lbl);
	st[sidx].st_type = lbl_type;
	st[sidx].st_value = val;
} /* end symtab_insert */

/*-----------------  S Y M T A B _ F I L E _ L O A D  -------------------------\

 Purpose: Taks the pointer to the symtab input file and loads the symtab

 Pre-Condition : <none> Follow Parameters

 Post-Condition: input file has been inserted to symtab 
		- duplicate labels are ignored

 Return: void

\-----------------------------------------------------------------------------*/
void symtab_file_load(struct st_entry *st, FILE **fp)
{
 	char line[INP_LENGTH + 1],
      	     lbl[NAME_LENGTH + 1],
	     temp[NAME_LENGTH + 1],
	     type,
	     *p;

	int  st_index,
	     isfound,
	     lctr = 0;

		  fgets(&line[0], INP_LENGTH, in_fptr); /* Absorbs First Line */
	while(fgets(&line[0], INP_LENGTH, *fp) != NULL)
	{
		if ((strcmp(line,"12345678901234567890\n")!= 0))
		{
			if ((p = strtok(line, DELIM)) != NULL)
				strcpy(lbl, p);
			if ((p = strtok(NULL, DELIM)) != NULL)
			{
				strcpy(temp, p);
				sscanf(temp, "%X", &lctr);
			}
			if ((p = strtok(NULL, DELIM)) != NULL)
				type = *p;
			st_index = symtab_hash(lbl);
			isfound = symtab_search(lbl, st, &st_index);
			if (isfound == FALSE)
			{
					  symtab_insert(lbl, type, st, st_index, lctr);
			}
		}
	}
} /* end symtab_file_load */

/*--------------  S Y M T A B _ R E G I S T E R _ L O A D  --------------------\

 Purpose: loads the register information into symtab - hard coded for pass2

 Pre-Condition : <none> Follow Parameters

 Post-Condition:
         registers and their values are in the symtab

 Return: void

\-----------------------------------------------------------------------------*/
void symtab_register_load(struct st_entry *st)
{
	int  st_index, j,
	     isfound,
	     reg_val[]  = { 0,   1,   2,   8,    9,    3,   4,   5,   6};

	char *reg_lbl[] = {"A", "X", "L", "PC", "SW", "B", "S", "T", "K"};

	for (j = 0; j < REG_TOTAL; j++)
	{
		st_index = symtab_hash(reg_lbl[j]);
		isfound = symtab_search(reg_lbl[j], st, &st_index);
		if (isfound == FALSE)
		{
			symtab_insert(reg_lbl[j],REGISTER, st, st_index, 
			reg_val[j]);
		}
	}
} /* end symtab_register_load */

/*------------------------  S Y M T A B _ W R I T E   -------------------------\

 Purpose: Writes the symtab contents to a file

 Pre-Condition : <none> Follow Parameters

 Post-Condition: symtab is written to file

 Return: void

\-----------------------------------------------------------------------------*/
void symtab_write(struct st_entry *st, FILE **fp)
{
	int j = 0;

	fprintf(*fp, "\n        The Symbol Table\n");
	fprintf(*fp, "Hash Index   Name        Value   Type\n");
	while(j < HT_SIZE)
	{
		if  (st[j].st_name[0] != '\0')
		{
			fprintf(*fp,"%5d        %-13s%-9.4X%c\n",
				 j, st[j].st_name, st[j].st_value, 
				 st[j].st_type);
		} /* end if */
		j++;
	}  /* end while */
} /* end symtab_write */

/*--------------------  END SYMBOL TABLE  FUNCTIONS  -----------------------*/
/* submitted by Bettina */

